
/* *************************************************************************************
* IR API:
*   DLL:     IrLrnApi.dll  & Uart.dll
*   Header:  IrLrnApi.h
*   Lib:     IrLrnApi.lib
*
* Description:
* IR API is the statically-linked Regular DLL designed for Windows XP ~ Windows 11
* desktop applicatins. Users can apply it to develop their own applications for Universal
* IR learning remote modules by the programming language such as Visual C++, Visual C#.NET
* and Visual Basic.NET.
*
* Release Note:
* v2.2.0.0 [2022-June-02]
*   IR API Library for IR-210,IR-712A and IR-712-MTCP.
*
* **************************************************************************************** */
#ifndef __IRLRNAPI_H__
#define __IRLRNAPI_H__

#ifdef __cplusplus
extern "C" {
#endif

#ifdef DLL_EXPORTS
  #ifdef  __cplusplus
    #define EXPORTS extern "C" __declspec (dllexport)
  #else
    #define EXPORTS __declspec (dllexport)
  #endif
#else
  #ifdef  __cplusplus
    #define EXPORTS extern "C" __declspec (dllimport)
  #else
    #define EXPORTS __declspec (dllimport)
  #endif
#endif

//
// Return Code
//

#define    IR_E_OK                              0x00	/* No errors. The function runs successfully.                                                                  */
#define    IR_E_COM_STATUS_AVAIL                0x01	/* The COM port is available and not open.                                                                     */
#define    IR_E_IRCMD_DATA_EMPTY                0x00	/* NO IR data exist for the specified IR cmd number in the library buffer.                                         */
#define    IR_E_IRCMD_DATA_EXIST                0x01	/* IR data is available for the specified IR cmd number in the library buffer.                                            */
#define    IR_E_RESET_OK                        0x02	/* Reset IR module sucessfully.                                                                                */  

#define    IR_E_FUNC_ERR                        0x11	/* Function error.                                                                                             */
#define    IR_E_COM_STATUS_ERR                  0x12	/* The COM port is not available or is already open.                                                           */
#define    IR_E_OPEN_COM_ERR                    0x13	/* Open COM port error                                                                                         */
#define    IR_E_ILL_BAUD_IDX                    0x14	/* Illegal baud rate index                                                                                     */
#define    IR_E_ILL_DATA_BIT                    0x15	/* Illegal data bits                                                                                           */
#define    IR_E_ILL_STOP_BIT                    0x16	/* Illegal stop bits                                                                                           */
#define    IR_E_ILL_PARITY                      0x17	/* Illegal parity                                                                                              */
#define    IR_E_MODNAME_INVALID                 0x18	/* Get invalid module name.                                                                                    */
#define    IR_E_SEND_MBCMD_ERR                  0x19	/* Error in sending/receiving Modbus command.                                                                  */
#define    IR_E_SEND_MBCMD_FLASH_ERR            0x1A	/* Error in sending/receiving Modbus command to write data to Flash.                                           */
#define    IR_E_ILL_SUMMARY_LEN                 0x1B	/* Illegal length of the array for IR command summary (should be 13)                                           */
#define    IR_E_DEVICE_QTY_ERR                  0x1C	/* The device quantity exceeds the maximum.                                                                    */
#define    IR_E_IRCMD_TOTAL_QTY_ERR             0x1D	/* The element "Total quantity of IR commands" in the IR cmd summary exceeds the maximum.                      */
#define    IR_E_IRCMD_SUM_QTY_ERR               0x1E	/* The sum of all device's IR command quantity exceeds the maximum.                                            */
#define    IR_E_ILL_MODULE_INDEX                0x1F	/* Illegal module index                                                                                        */
#define    IR_E_ILL_IRCMD_NO                    0x20	/* Illegal IR command number                                                                                   */
#define    IR_E_DEVICE_NAME_ARR_UNDERSIZE       0x21	/* The length of the device name array is too small as an argument to a function.                              */
#define    IR_E_IRCMD_NAME_ARR_UNDERSIZE        0x22	/* The length of the IR cmd name array is too small as an argument to a function.                              */
#define    IR_E_DEVICE_NAME_ARR_OVERSIZE        0x23	/* The length of the device name is too large as an argument to a function.                                    */
#define    IR_E_IRCMD_NAME_ARR_OVERSIZE         0x24	/* The length of the IR cmd name is too large as an argument to a function.                                    */
#define    IR_E_LRNCMD_LEN_EXCEED_MAX           0x25	/* The length of the IR cmd learning data exceeds the maximum.                                                 */
#define    IR_E_RESET_COM_ERR                   0x26	/* COM error in sending reset message to the IR module.                                                        */ 
#define    IR_E_RESET_MODULE_FAIL               0x27	/* Resetting the IR module is not successful.												                   */
#define    IR_E_IRCMD_TOTALQTY_NOMATCH			0x28	/* The totl quantity of IR cmds do not match the sum of IR cmds in all devices in the array of IR cmd summary  */
#define    IR_E_OPEN_FILE_ERROR					0x29	/* Open file error																			                   */ 
#define    IR_E_CLOSE_FILE_ERROR				0x2A	/* Close file error																			                   */
#define    IR_E_IR_FILE_FORMAT_ERR				0x2B	/* Format error in IR learning data file													                   */
#define    IR_E_SOCKET_INVALID					(-1)	/* SOCKET Error																				                   */
#define    IR_E_SOCKET_BIND_ERR					0x2C	/* Error in binding socket with the network address of NIC                                                     */
#define    IR_E_SOCKET_CONN_TOUT				0x2D	/* Connection timeout by Modbus TCP                                                                            */
#define    IR_E_SOCKET_RECV_TOUT				0x2E	/* Socket receive timeout                                                                                      */

// This structure colllects basic setting parameters of IR modules with serial communication interface.
#pragma pack(1)
	typedef struct
	{
		unsigned char  ucBaudrateIndex;  // Baud rate Index. [6]:9600 bps, [10]:115200 bps 
		unsigned char  ucData;           // Data bits  always => 8
		unsigned char  ucParity;         // Parity  0=>None  1=>Odd  2=>Even
		unsigned char  ucStop;           // Stop bits.  0=>1 stopbit, 2=>2 stopbits
		unsigned char  ucNetID;          // Modbus Net ID of IR modules ( 1 ~ 247 ) 
		unsigned short  wMBRespDlyT;     // Modbus response delay time (ms), default = 3 ms
		unsigned short  wGapTime;        // Gap time (ms), default= 72 ms, range = 72 ~ 200 ms

	}BasicSetParams;
#pragma pack()


// This structure colllects basic setting parameters of IR modules with Ethernet interface. (Modbus TCP/UDP).
#pragma pack(1)
	typedef struct
	{
		unsigned long  ulIPAddr;    // IP Addres.   192.168.0.1 = > 0xC0A80001
		unsigned long  ulMaskAddr;	// Subnet Mask Addres.  255.255.0.0 = > 0xFFFF0000
		unsigned long  ulGWAddr;    // Gateway address. 192.168.0.254 = > 0xC0A800FE 
		unsigned long  ulIdleTOut;  // Idle communication timeout (mimnute)
		unsigned char  ucNetID;     // Modbus Net ID of IR modules ( 1 ~ 247 ) 
		unsigned char  ucAddrType;  // Address type. 0: static  1: DHCP
		unsigned short usGapTime;   // Gap time (ms), default= 72 ms, range = 72 ~ 200 ms
	}BasicSetParamsMTCP;
#pragma pack()


// This structure colllects the information of a searched IR module with Ethernet interface.
#pragma pack(1)
	typedef struct
	{
		unsigned __int64 dwlMacAddr; // MAC address 1a:2b:3c:4d:5e:6f => 0x00001A2B3C4D5E6F
		unsigned long  ulIPAddr;     // IP Addres.   192.168.0.1 => 0xC0A80001
		unsigned long  ulMaskAddr;	 // Mask Addres.  255.255.0.0 => 0xFFFF0000
		unsigned long  ulGWAddr;     // Gateway address. 192.168.0.254 => 0xC0A800FE 
		unsigned long  ulFwVer;      // Firmware version of the module. v1.2.3 => 0x00010203 
		unsigned long  ulIdleTOut;   // Idle communication timeout (unit: mimnute, 0 ~ 65535)
		unsigned char  ucNetID;      // Modbus Net ID of IR modules ( 1 ~ 247 ) 
		unsigned char  ucAddrType;   // Address type. 0: static  1: DHCP 
		unsigned char  ucHwMode;	 // Mode: OP(=0) or FW/Init (=1) mode
		unsigned char  ucModIndex;   // Module Index.   0x21 => "IR-712-MTCP"	
	}SearchRespEth;					 
#pragma pack()


	typedef int tHandle;				 // Socket handle for Modbus TCP/UDP


	/* ***************************************************************
	*
	* IR API Function Declaration
	*
	****************************************************************** */


	/**********************************************************************************************
	* Function Name: IR_GetAPIVersion
	*
	* Description: Get the version number of the IR API.
	*
	* Parameter: None
	*
	* Return: Version of IR API. Three bytes: Major, Minor, Revision, build.
	*         E.g. 0x01020300 => v1.2.3.0
	*
	***********************************************************************************************/
	EXPORTS unsigned long __stdcall IR_GetAPIVersion(
		void
		);


	/**********************************************************************************************
	* Function Name: IR_GetAPIDate
	*
	* Description: Get the date of the IR API.
	*
	* Parameter: None
	*
	* Return: Date of IR API.
	*         E.g. 0x07DF080C for 2015-Aug-12
	*
	***********************************************************************************************/
	EXPORTS unsigned long __stdcall IR_GetAPIDate(
		void
		);


	/* *********************************************************************************************
	* Function Name: IR_GetComPortStatus
	*
	* Description:
	*   Check if the COM port is in use or not.
	*
	* Parameter:
	*   [in]ucPort: COM port number
	*
	* Return:
	*   IR_E_COM_STATUS_AVAIL: COM port is available and not open.
	*   IR_E_COM_STATUS_ERR: COM port is not available or is open.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_GetComPortStatus(
		unsigned char ucPort
		);


	/* *********************************************************************************************
	* Function Name: IR_OpenCom
	*
	* Description:
	*   Open a COM port of the host (Modbus Master).
	*
	* Parameter:
	*   [in]ucPort: COM port number
	*   [in]ucBaudIndex: baudrate index [6] ~ [10] => 9600 bps ~ 115200 bps
	*           supported baudrate(bps): 9600,19200,38400,57600,115200
	*   [in]ucData: The data bits is always 8.
	*   [in]ucParity: Parity. 0 => None Parity), 1 => Odd Parity, 2 => Even Parity
	*   [in]ucStop: Stop bits. 1 => 1 stop bit, 2 => 2 stop bit
	*
	* Return:
	*   IR_E_OK: No errors. Open Com OK
	*   IR_E_ILL_BAUD_IDX: illegal baudrate index
	*	IR_E_ILL_DATA_BIT: illegal data bits
	*	IR_E_ILL_PARITY: illegal parity
	*	IR_E_ILL_STOP_BIT: illegal stop bits
	*   IR_E_OPEN_COM_ERR: Open COM error
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_OpenCom(
		unsigned char ucPort,
		unsigned char ucBaudIndex,
		unsigned char ucData,
		unsigned char ucParity,
		unsigned char ucStop
		);

	
	/* *********************************************************************************************
	* Function: IR_CloseCom
	*
	* Description:
	*   Close COM port of the host (Modbus master).
	*
	* Parameter:
	*   [in]ucPort: COM port number 1~255
	*
	* Return:
	*   IR_E_OK:  No Error
	*   IR_E_FUNC_ERR: Other function errors
	*
	* **********************************************************************************************/
	EXPORTS int __stdcall IR_CloseCom(
		unsigned char ucPort
		);


	/* *********************************************************************************************
	* Function: IR_SetBasicSetting
	*
	* Description:
	*   Set the basic settings to  the IR remote module.
	*   Take care the times to call this function in programming because there are max 100000 times
	*   to write basic setings to the flash memory.
	*
	* Parameter:
	*   [in]ucPort: COM port number
	*   [in]ucNetID: Modbus ID of the IR module (1 ~ 247)
	*   [in]SBasicSetting: BasicSetParams structure variable for the bsaic settings of the IR learning
	*                      remote module.
	*
	* Return:
	*   IR_E_OK:  No Error
	*   IR_E_ILL_BAUD_IDX: illegal baudrate index
	*	IR_E_ILL_DATA_BIT: illegal data bits
	*	IR_E_ILL_PARITY: illegal parity
	*	IR_E_ILL_STOP_BIT: illegal stop bits
	*   IR_E_SEND_MBCMD_FLASH_ERR: Error in sending Modbus command to write data to Flash.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_SetBasicSetting(
		unsigned char ucPort,
		unsigned char ucNetID,
		/*[in]*/BasicSetParams SBasicSetting
		);


	/* *********************************************************************************************
	* Function: IR_GetBasicSetting
	*
	* Description:
	*   Get the basic settings from the IR learning remote module.
	*
	* Parameter:
	*   [in]ucPort: COM port number
	*   [in]ucNetID: Modbus ID of the IR module (1 ~ 247)
	*   [out]pSBasicSetting: Structure variable of the bsaic settings of the IR learning remote module.
	*
	* Return:
	*   IR_E_OK:  No Error
	*   IR_E_ILL_BAUD_IDX:  illegal baudrate
	*	IR_E_ILL_DATA_BIT:  illegal data bits
	*	IR_E_ILL_PARITY:  illegal parity
	*	IR_E_ILL_STOP_BIT:  illegal stop bits
	*   IR_E_SEND_MBCMD_ERR:  Error in reading basic settings from the IR module.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_GetBasicSetting(
		unsigned char ucPort,
		unsigned char ucNetID,
		/*[out]*/BasicSetParams* pSBasicSetting
		);


	/* *********************************************************************************************
	* Function: IR_ChkModuleName
	*
	* Description:
	*   Get model name of the IR remote module.
	*
	* Parameter:
	*   [in]ucPort: COM port number
	*   [in]ucNetID: Modbus ID of the IR module (1 ~ 247)
	*   [out]pwModuleIndex: Module index representing for specific module type(name),
	*         e.g. 0x01=>"IR210",
	*              0x11=>"IR712A".
	*
	* Return:
	*   IR_E_OK:  No Error
	*   IR_E_SEND_MBCMD_ERR:  Error in sending/receiving Modbus command.
	*   IR_E_MODNAME_INVALID:  Get invalid module name.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_ChkModuleName(
		unsigned char ucPort,
		unsigned char ucNetID,
		unsigned short *pwModuleIndex
		);


	/* *********************************************************************************************
	* Function: IR_LrnModeSet
	*
	* Description:
	*   Enable/disable IR learning mode.
	*
	* Parameter:
	*   [in]ucPort: COM port number
	*   [in]ucNetID: Modbus Net ID: 1 ~247
	*   [in]ucMode: Learning mode. 0 => learn off, 1=> learn on.
	*
	* Return:
	*	IR_E_OK: No errors.
	*	IR_E_SEND_MBCMD_ERR: Error in sending/receiving Modbus command.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_LrnModeSet(
		unsigned char ucPort,
		unsigned char ucNetID,
		unsigned char ucMode
		);


	/* *********************************************************************************************
	* Function: IR_IsIRLearnOK
	*
	* Description:
	*   Show the status of IR learnig by receiving the learning OK message once after learning
	*   an IR command is finished.
	*
	* Parameter:
	*   [in]ucPort: COM port number
	*   [out]*pusIsOK: 0: IR learning not OK, 1: IR learning OK
	*   [in]usTimeout: receive timeout (ms)
	*
	* Return:
	*	IR_E_OK: No errors.
	*   IR_E_COM_STATUS_ERR: The COM port is not available or is open.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_IsIRLearnOK(
		unsigned char ucPort,
		unsigned short *pusIsOK,
		unsigned short usTimeout
		);


	/* *********************************************************************************************
	* Function: IR_SetIRCmdSummary
	*
	* Description:
	*   Write the IR command summary from the library buffer to the flash memory
	*   of the IR learning module.
	*   !Note: IR_SetIRCmdSummaryBuffer() should be called first.
	*
	* Parameter:
	*   [in]ucPort: COM port number
	*   [in]usNetID: Modbus Net ID of the IR module.
	*   [in]usIrModuleIndex: Index of IR learning remote module
	*                             0x01: IR-210
	*                             0x11: IR-712A
	*
	* Return:
	*	IR_E_OK: No errors.
	*	IR_E_FUNC_ERR:  Function error.
	*   IR_E_DEVICE_QTY_ERR:  The device quantity exceeds the maximum.
	*	IR_E_IRCMD_TOTAL_QTY_ERR:  The device quantity setting exceeds the maximum.
	*	IR_E_IRCMD_SUM_QTY_ERR:  The element "Total quantity of IR commands" in the IR cmd summary
	*                            exceeds the maximum.
	*   IR_E_ILL_MODULE_INDEX:  Illegal module index.
	*   IR_E_SEND_MBCMD_ERR:  Error in sending/receiving Modbus command.
	*   IR_E_SEND_MBCMD_FLASH_ERR:  Error in sending/receiving Modbus command to write data to Flash.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_SetIRCmdSummary(
		unsigned char ucPort,
		unsigned char usNetID,
		unsigned short usIrModuleIndex
		);


	/* *********************************************************************************************
	* Function: IR_GetIRCmdSummary
	*
	* Description:
	*   Read IR command summary from the IR learning module.
	*
	* Parameter:
	*   [in]ucPort: COM port number
	*   [in]ucNetID: Modbus Net ID of the IR module.
	*   [in]usIrModuleIndex: Index of IR learning remote module
	*                             0x01: IR-210
	*                             0x11: IR-712A
	*   [out]*dataArr: data array for IR command summary. Array length = 13
	*                     where dataArr[0]: IR device quantity (0 ~ 11)
	*                           dataArr[1]: Total quantity of IR commands ( 0 ~ 224 )
	*                           dataArr[2]: IR command quantity of device#1
	*                           dataArr[3]: IR command quantity of device#2
	*                             .....         .........................
	*                           dataArr[N+1]: IR command quantity of device#N , where N<=11
	*
	*   [in]usInElementCnt: Length of the dataArr.  Should be at least 13.
	*
	*
	* Return:
	*   IR_E_OK: No errors.
	*   IR_E_ILL_SUMMARY_LEN:  Illegal length of the array for IR command summary
	*   IR_E_FUNC_ERR:  Function error.
	*	 IR_E_IRCMD_TOTAL_QTY_ERR:  The device quantity setting exceeds the maximum.
	*   IR_E_IRCMD_SUM_QTY_ERR:  The element "Total quantity of IR commands" in the IR cmd summary
	*                            exceeds the maximum.
	*   IR_E_ILL_MODULE_INDEX:  Illegal module index.
	*   IR_E_SEND_MBCMD_ERR:  Error in sending/receiving Modbus command.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_GetIRCmdSummary(
		unsigned char ucPort,
		unsigned char ucNetID,
		unsigned short usIrModuleIndex,
		unsigned short *dataArr,
		unsigned short usInElementCnt
		);


	/* **************************************************************************************************
	* Function: IR_GetIRCmdAfterLearn
	*
	* Description:
	*	 Get one IR learning command data(without device name & IR cmd name) from the IR learning
	*   remote module to the buffer of the API library right after learning an IR command successfully.
	*
	* Parameter:
	*   [in]ucPort: COM port number
	*   [in]usNetID: Modbus Net ID of the IR module.
	*   [in]usIrCmdNumber: IR command number for allocating one IR learning command data
	*                      in the learning data buffer
	*
	* Return:
	*   IR_E_OK:  No errors.
	*   IR_E_ILL_IRCMD_NO:  Illegal IR command number.
	*   IR_E_SEND_MBCMD_ERR: No response
	*
	* ************************************************************************************************** */
	EXPORTS int __stdcall IR_GetIRCmdAfterLearn(
		unsigned char ucPort,
		unsigned char ucNetID,
		unsigned short usIrCmdNumber
		);


	/* *********************************************************************************************
	* Function: IR_ChkIrDataExistInBuffer
	*
	* Description:
	*   Check if there are IR learning data for the specified IR cmd number in the buffer
	*
	* Parameter:
	*   [in]usIrCmdNumber: IR command number for allocating one IR learning command data
	*                      in the learning data buffer
	*
	* Return:
	*	IR_E_IRCMD_DATA_EMPTY: No IR learning data for the specified IR command number in the buffer.
	*	IR_E_IRCMD_DATA_EXIST: There are IR learning data for the specified IR command number in the buffer.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_ChkIrDataExistInBuffer(
		unsigned short usIrCmdNumber
		);


	/* *********************************************************************************************
	* Function: IR_GetIrDevCmdNameInBuffer
	*
	* Description:
	*   Get the IR device name byte array and the IR cmd name byte array corrsponding to
	*   the IR cmd number in the API buffer.
	*
	* Parameter:
	*   [in]usIrCmdNumber: IR command number for allocating one IR learning command data
	*                           in the learning data buffer
	*   [out]*ucIrDevNameStr: Byte array for the IR device Name string. Byte Array Max length
	*                         = 100 bytes (50 words)
	*   [in]usDevNameByteArrLen: Byte length of the IR devie name array (ucIrDevNameArr). Max length = 100 bytes
	*                         (50 words).
	*   [out]*usDevNameByteArrOutLen: Byte length of the IR devie name string without terminator.
	*   [out]*ucIrCmdNameStr: String of the IR Command Name. Byte Array Max length
	*                         = 102 bytes (51 words)
	*   [in]usIrCmdNameByteArrLen: Byte length of the IR command name Array. Max length = 102 bytes
	*                         (51 words).
	*   [out]*usIrCmdNameByteArrOutLen: Byte length of the IR command name string without terminator.
	*
	* Return:
	*   IR_E_OK: No errors.
	*   IR_E_DEVICE_NAME_ARR_UNDERSIZE: The size of the device name array variable is too small
	*                                   as an argument of a function.
	*   IR_E_IRCMD_NAME_ARR_UNDERSIZE: The size of the IR cmd name array variable is too small
	*                                    as an argument of a function.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_GetIrDevCmdNameInBuffer(
		unsigned short usIrCmdNumber,
		unsigned char *ucIrDevNameStr,         // Byte array with Unicode value
		unsigned short usDevNameByteArrLen,    // Device name Byte array length
		unsigned short *usDevNameByteArrOutLen,   // Actual length(unit byte) of device name string without teminator.
		unsigned char *ucIrCmdNameStr,         // Byte array with Unicode value
		unsigned short usIrCmdNameByteArrLen,  // IR commmand name Byte array length
		unsigned short *usIrCmdNameByteArrOutLen    // Actual length(unit byte) of IR cmd name string without teminator.
		);


	/* *********************************************************************************************
	* Function: IR_SetIrDevCmdNameInBuffer
	*
	* Description:
	*   Set the IR device name string and the IR cmd name string corrsponding to
	*   the IR cmd number in the buffer
	*
	* Parameter:
	*   [in]usIrCmdNumber: IR command number for allocating one IR learning command data
	*                      in the learning data buffer
	*   [in]*ucIrDevNameStr: Byte array for the IR device Name string.
	*                         Byte Array Max length = 100 bytes (50 words)
	*   [in]usDevNameUcLen: Length of the byte array for the IR devie name string.
	*                       Max length = 100 bytes (50 words).
	*   [in]*ucIrCmdNameStr: Byte array for the IR Command Name string. Byte Array
	*                        Max length = 100 bytes (50 words )
	*   [in]usIrCmdNameUcLen: Length of the byte array for the IR command name string.
	*                         Max length = 100 bytes (50 words).
	*
	*
	* Return:
	*   IR_E_OK: No errors.
	*   IR_E_DEVICE_NAME_ARR_OVERSIZE:  The length of the device name is too large as an argument to a function.
	*   IR_E_IRCMD_NAME_ARR_OVERSIZE:  The length of the IR cmd name is too large as an argument to a function.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_SetIrDevCmdNameInBuffer(
		unsigned short usIrCmdNumber,
		unsigned char *ucIrDevNameStr,
		unsigned short usDevNameUcLen,
		unsigned char *ucIrCmdNameStr,
		unsigned short usIrCmdNameUcLen
		);


	/* *********************************************************************************************
	 * Function: IR_EmitIrSignal
	 *
	 * Description:
	 *	 Request the IR module to emit the IR command stored in the flash memory.
	 *   Emit IR signal with selected IR command number and IR output channels.
	 *
	 * Parameter:
	 *   [in]ucPort: COM port number
	 *   [in]usNetID: Modbus Net ID of the IR module.
	 *   [in]usIrCmdNumber: IR command number for allocating one IR learning command data
	 *                           in the learning data buffer
	 *   [in]ucIrOutputCh: IR output channels.  Use IR output ch1 & ch2 => value = 0x03 (0000 0011(binary))
	 *
	 * Return:
	 *   IR_E_OK: No errors.
	 *   IR_E_ILL_IRCMD_NO:  Illegal IR command number.
	 *   IR_E_SEND_MBCMD_ERR: Error in sending/receiving Modbus command.
	 *
	 * ********************************************************************************************* */
	EXPORTS int __stdcall IR_EmitIrSignal(
		unsigned char ucPort,
		unsigned char ucNetID,
		unsigned short usIrCmdNumber,
		unsigned char ucIrOutputCh
		);


	/* *********************************************************************************************
	 * Function: IR_RunCommand
	 *
	 * Description:
	 *   Send an IR command learning data to the IR module and emit that IR command.
	 *
	 * Parameter:
	 *   [in]ucPort: COM port number
	 *   [in]usNetID: Modbus Net ID of the IR module.
	 *   [in]usIrCmdNumber: IR command number for allocating one IR learning command data
	 *                      in the learning data buffer
	 *   [in]ucIrOutputCh: IR output channels
	 *
	 *
	 * Return:
	 *   IR_E_OK: No errors.
	 *   IR_E_IRCMD_DATA_EMPTY:  NO IR data exists for the specified IR cmd number in the API buffer.
	 *   IR_E_ILL_IRCMD_NO:  Illegal IR command number.
	 *   IR_E_LRNCMD_DATA_EXCEED_MAX:  The length of the IR cmd learning data exceeds the maximum.
	 *   IR_E_SEND_MBCMD_ERR: Error in sending/receiving Modbus command.
	 *
	 * ********************************************************************************************* */
	EXPORTS int __stdcall IR_RunCommand(
		unsigned char ucPort,
		unsigned char ucNetID,
		unsigned short usIrCmdNumber,
		unsigned char ucIrOutputCh
		);


	/* *********************************************************************************************
	 * Function: IR_DLOneIRDataToModule
	 *
	 * Description:
	 *	 Download one IR cmd data (:= IR device name + IR cmd name + IR cmd learning data) from the
	 *   API buffer to the IR Module.
	 *
	 * Parameter:
	 *   [in]ucPort: COM port number
	 *   [in]usNetID: Modbus Net ID of the IR module.
	 *   [in]usIrCmdNumber: IR command number for allocating one IR learning command data
	 *                      in the learning data buffer
	 *
	 * Return:
	 *   IR_E_OK: No errors.
	 *   IR_E_ILL_IRCMD_NO: Illegal IR command number
	 *   IR_E_SEND_MBCMD_ERR:  Error in sending/receiving Modbus command.
	 *   IR_E_SEND_MBCMD_FLASH_ERR:  Error in sending/receiving Modbus command to write data to Flash.
	 *   IR_E_LRNCMD_LEN_EXCEED_MAX:  The length of the IR cmd learning data exceeds the maximum.
	 *
	 * ********************************************************************************************* */
	EXPORTS int __stdcall IR_DLOneIRDataToModule(
		unsigned char ucPort,
		unsigned char ucNetID,
		unsigned short usIrCmdNumber
		);


	/* *********************************************************************************************
	 * Function: IR_LDOneIRDataToAPI
	 *
	 * Description:
	 *	 Load one IR data (IR device name + IR cmd name + IR learning) from an IR module to the API
	 *   buffer.
	 *
	 * Parameter:
	 *   [in]ucPort: COM port number
	 *   [in]usNetID: Modbus Net ID of the IR module.
	 *   [in]usIrCmdNumber: IR command number for allocating one IR learning command data
	 *                      in the learning data buffer
	 *
	 * Return:
	 *   IR_E_OK: No errors.
	 *   IR_E_ILL_IRCMD_NO: Illegal IR command number
	 *   IR_E_SEND_MBCMD_ERR:  Error in sending/receiving Modbus command.
	 *
	 * ********************************************************************************************* */
	EXPORTS int __stdcall IR_LDOneIRDataToAPI(
		unsigned char ucPort,
		unsigned char ucNetID,
		unsigned short usIrCmdNumber
		);


	/* *********************************************************************************************
	 * Function: IR_ResetIRModule
	 *
	 * Description:
	 *	Reset Communication settings of the IR module. Wait block three seconds.
	 *
	 * Parameter:
	 *   ucPort[in]: COM port number
	 *
	 * Return:
	 *   IR_E_RESET_OK: Reset IR module successfully.
	 *   IR_E_RESET_COM_ERR:  COM error in sending reset message to the IR module.
	 *                             Receive timeout or COM error.
	 *   IR_E_RESET_MODULE_FAIL:  Failed to reset the IR module.
	 *
	 * ********************************************************************************************* */
	EXPORTS int __stdcall IR_ResetIRModule(
		unsigned char ucPort
		);


	/* *********************************************************************************************
	* Function: IR_GetModbusRecvTimeout
	*
	* Description:
	*	 Get the receive timeout (ms) for the command to the IR module. Default value = 50 ms
	*
	* Parameter:
	*   None
	*
	* Return:
	*   Modbus receive timeout (ms)
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_GetModbusRecvTimeout(
		void
		);


	/* *********************************************************************************************
	 * Function: IR_SetModbusRecvTimeout
	 *
	 * Description:
	 *   Set the receive timeout (ms) for the command to the IR module. Default value = 50 ms
	 *
	 * Parameter:
	 *   [in]recvTout: Receive timeout for the Modbus response after sending a Modbus request.
	 *
	 * Return:
	 *   None
	 *
	 * ********************************************************************************************* */
	EXPORTS void __stdcall IR_SetModbusRecvTimeout(
		unsigned short recvTout
		);


	/* *********************************************************************************************
	* Function: IR_ClearOneIRLrnDataInAPI
	*
	* Description:
	*	 Clear one IR command learning data in the API buffer.
	*    (IR command name will not be cleared.)
	*
	* Parameter:
	*   [in]usIrCmdNumber: IR command number for allocating one IR learning command data
	*                      in the learning data buffer
	*
	* Return:
	*   IR_E_OK: No errors.
	*   IR_E_FUNC_ERR: Function error
	*   IR_E_ILL_IRCMD_NO: Illegal IR command number
	*   IR_E_ILL_MODULE_INDEX:  Illegal module index
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_ClearOneIRLrnDataInAPI(
		unsigned short usIrCmdNumber,
		unsigned short usIrModuleIndex
		);


	/* *********************************************************************************************
	* Function: IR_ClearOneIRDeiveCmdNameInAPI
	*
	* Description:
	*	 Clear the IR device name and IR command name corrsponding to the IR command number in the API buffer.
	*
	* Parameter:
	*   [in]usIrCmdNumber: IR command number for allocating one IR learning command data
	*                      in the learning data buffer
	*   [in]usIrModuleIndex: Index of IR learning remote module
	*                             0x01: IR-210
	*                             0x11: IR-712A
	*
	* Return:
	*   IR_E_OK: No errors.
	*   IR_E_FUNC_ERR: Function error
	*   IR_E_ILL_IRCMD_NO: Illegal IR command number
	*   IR_E_ILL_MODULE_INDEX:  Illegal module index
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_ClearOneIRDeiveCmdNameInAPI(
		unsigned short usIrCmdNumber,
		unsigned short usIrModuleIndex
		);


	/* *********************************************************************************************
	* Function: IR_GetIRCmdSummaryBuffer
	*
	* Description:
	*   Read IR command summary from the buffer of the IR API Library.
	*
	* Parameter:
	*   [in]usIrModuleIndex: Index of IR learning remote module
	*                             0x01: IR-210
	*                             0x11: IR-712A
	*                             0x12: IR-712-MTCP
	*   [out]*dataArr: data array for IR command summary. Array length = 13
	*                     where dataArr[0]: IR device quantity (1 ~ 11)
	*                           dataArr[1]: Total quantity of IR commands ( 1 ~ 224 )
	*                           dataArr[2]: IR command quantity of device#1
	*                           dataArr[3]: IR command quantity of device#2
	*                             .....         .........................
	*                           dataArr[N+1]: IR command quantity of device#N , where N<=11
	*
	*   [in]usInElementCnt: Length of the dataArr.  Should be at least 13.
	*
	*
	* Return:
	*   IR_E_OK: No errors.
	*   IR_E_ILL_SUMMARY_LEN:  Illegal length of the array(dataArr) for IR command summary
	*   IR_E_FUNC_ERR:  Function error.
	*   IR_E_DEVICE_QTY_ERR: The device quantity exceeds the maximum.
	*	IR_E_IRCMD_TOTAL_QTY_ERR:  The device quantity setting exceeds the maximum.
	*   IR_E_IRCMD_SUM_QTY_ERR:  The element "Total quantity of IR commands" in the IR cmd summary
	*                            exceeds the maximum.
	*   IR_E_ILL_MODULE_INDEX:  Illegal module index.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_GetIRCmdSummaryBuffer(
		unsigned short usIrModuleIndex,
		unsigned short *dataArr,
		unsigned short usInElementCnt
		);


	/* *********************************************************************************************
	* Function: IR_SetIRCmdSummaryBuffer
	*
	* Description:
	*	  Set the IR command summary in the buffer of the API library.
	*
	* Parameter:
	*   [in]usIrModuleIndex: Index of IR learning remote module
	*                             0x01: IR-210
	*                             0x11: IR-712A
	*                             0x21: IR-712-MTCP
	*   [in]*dataArr: (Unsigned short) data array for IR command summary. Array length = 13
	*                     where dataArr[0]: IR device quantity (1 ~ 11)
	*                           dataArr[1]: Total quantity of IR commands ( 1 ~ 224 )
	*                           dataArr[2]: IR command quantity of device#1
	*                           dataArr[3]: IR command quantity of device#2
	*                             .....         .........................
	*                           dataArr[N+1]: IR command quantity of device#N , where N<=11
	*
	*   [in]usInElementCnt: Length of the dataArr. Should contain 13 elements exactly.
	*
	* Return:
	*   IR_E_OK: No errors.
	*   IR_E_ILL_SUMMARY_LEN:  Illegal length of the array for IR command summary (should be 13)
	*	IR_E_FUNC_ERR:  Function error.
	*   IR_E_DEVICE_QTY_ERR:  The device quantity exceeds the maximum.
	*	IR_E_IRCMD_TOTAL_QTY_ERR:  The device quantity setting exceeds the maximum.
	*	IR_E_IRCMD_SUM_QTY_ERR:  The element "Total quantity of IR commands" in the IR cmd summary
	*                            exceeds the maximum.
	*   IR_E_ILL_MODULE_INDEX:  Illegal module index.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_SetIRCmdSummaryBuffer(
		unsigned short usIrModuleIndex,
		unsigned short *dataArr,
		unsigned short usInElementCnt
		);


	/* *********************************************************************************************
	* Function: IR_LoadIRCmdsFromFile
	*
	* Description:
	*	  Load IR commands from an IR learning data file(*.ird) to the IR API library.
	*
	* Parameter:
	*   [in]filepath: A char array for the full path of an IR learning data file.
	*
	*
	* Return:
	*   IR_E_OK: No errors.
	*	  IR_E_OPEN_FILE_ERROR: Open file error.
	*	  IR_E_IR_FILE_FORMAT_ERR: Format error in the IR learning data file.
	*	  IR_E_CLOSE_FILE_ERROR: Error in closing file.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_LoadIRCmdsFromFile(
		char* filepath
		);


	/* *********************************************************************************************
	* Function: IR_SaveIRCmdsToFile
	*
	* Description:
	*	  Save the IR commands in the IR API library to an IR learning data file(*.ird).
	*
	* Parameter:
	*   [in]filepath: A char array for the full path of an IR learning data file.
	*				  The existing file with the same filename will be overwritten without warning.
	*
	*
	* Return:
	*   IR_E_OK: No errors.
	*	  IR_E_IRCMD_TOTALQTY_NOMATCH: The total quantity of IR commands do not match that in the
	*                                IR command summary.
	*	  IR_E_OPEN_FILE_ERROR: Error in opening fil.
	*	  IR_E_CLOSE_FILE_ERROR: Error in closing file.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_SaveIRCmdsToFile(
		char* filepath
		);


	/* *********************************************************************************************
	* Function: IR_SetIRCmdSummary1
	*
	* Description:
	*   Write IR command summary to the flash memory of the IR learning module.
	*   The IR command summary is also written to the buffer of the API library.
	*
	* Parameter:
	*   [in]ucPort: COM port number
	*   [in]usNetID: Modbus Net ID of the IR module.
	*   [in]usIrModuleIndex: Index of IR learning remote module
	*                             0x01: IR-210
	*                             0x11: IR-712A
	*   [in]*dataArr: (Unsigned short) data array for IR command summary. Array length = 13
	*                     where dataArr[0]: IR device quantity (0 ~ 11)
	*                           dataArr[1]: Total quantity of IR commands ( 0 ~ 224 )
	*                           dataArr[2]: IR command quantity of device#1
	*                           dataArr[3]: IR command quantity of device#2
	*                             .....         .........................
	*                           dataArr[N+1]: IR command quantity of device#N , where N<=11
	*
	*   [in]usInElementCnt: Length of the dataArr. Should contain 13 elements exactly.
	*
	*
	* Return:
	*	IR_E_OK: No errors.
	*	IR_E_ILL_SUMMARY_LEN:  Illegal length of the array for IR command summary (should be 13)
	*	IR_E_FUNC_ERR:  Function error.
	*   IR_E_DEVICE_QTY_ERR:  The device quantity exceeds the maximum.
	*	IR_E_IRCMD_TOTAL_QTY_ERR:  The device quantity setting exceeds the maximum.
	*	IR_E_IRCMD_SUM_QTY_ERR:  The element "Total quantity of IR commands" in the IR cmd summary
	*                            exceeds the maximum.
	*   IR_E_ILL_MODULE_INDEX:  Illegal module index.
	*   IR_E_SEND_MBCMD_ERR:  Error in sending/receiving Modbus command.
	*   IR_E_SEND_MBCMD_FLASH_ERR:  Error in sending/receiving Modbus command to write data to Flash.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_SetIRCmdSummary1(
		unsigned char ucPort,
		unsigned char usNetID,
		unsigned short usIrModuleIndex,
		unsigned short *dataArr,
		unsigned short usInElementCnt
		);


	/* *********************************************************************************************
	* Function: IR_ChkNIC
	*
	* Description:
	*	Check the number of available network interfaces.
	*
	* Parameter:
	*   [out]iNiCnt: number of available network interface
	*
	* Return:
	*   void
	*
	* ********************************************************************************************* */
	EXPORTS void __stdcall IR_ChkNIC(
		int *iNiCnt
		);


	/* *********************************************************************************************
	* Function: IR_GetNICInfo
	*
	* Description:
	*	Get the description and IR address of available network interfaces.
	*
	* Parameter:
	*   [in]ucIndex: index(0-based) of the available NICs in the API buffer for getting the NIC info
	*   [out]szDescStr: NIC description string.
	*   [in]usDescStrLen: Length of szDescStr array. (number of unsigned char; maximum = 133)
	*   [out]ucRtnDecrLen: return the length of NIC description (excluding '\0').
	*   [out]ulIPAddr: IP address of the network interface card (e.g. 0xC0A8FF0F => 192.168.255.15)
	*
	* Return:
	*   void
	*
	* ********************************************************************************************* */
	EXPORTS void __stdcall IR_GetNICInfo(
		unsigned char ucIndex,
		unsigned char *szDescStr,
		unsigned short usDescStrLen,
		unsigned short *ucRtnDecrLen,
		unsigned long *ulIPAddr
		);


	/* *********************************************************************************************
	* Function: IR_EthSearchModules
	*
	* Description:
	*	Get the number of IR modules on the same local area network.
	*
	* Parameter:
	*   [in]ulIPAddrNIC: IP address of the network interface of a host PC.
	*   [out]pucModuleCnt: count of found modules
	*   [in]usWaitTimeout: timeout(ms) to wait for searching modules. (e.g. 2000 ms)
	*
	*
	* Return:
	*
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_EthSearchModules(
		unsigned long ulIPAddrNIC,
		unsigned char *pucModuleCnt,
		unsigned short usWaitTimeout
		);


	/* *********************************************************************************************
	* Function: IR_EthGetFoundModuleInfo
	*
	* Description:
	*	Get the information of the search modules after calling IR_EthSearchModules().
	*
	* Parameter:
	*   [in]usCntIndex: zero-based index of found modules. If found total module count is 3, available usCntIndex == 0, 1, 2.
	*   [out]ModuleInfo: output variable of struct SearchRespEth.
	*
	*
	* Return:
	*	None
	*
	* ********************************************************************************************* */
	EXPORTS void __stdcall IR_EthGetFoundModuleInfo(
		unsigned short usCntIndex,
		/*out*/SearchRespEth *ModuleInfo
		);





	// **********************************************************
	//  ************   Modbus/TCP communication
	// **********************************************************

	/* *********************************************************************************************
	* Function: IR_MTCP_TCPNew
	*
	* Description:
	*	  Initialize a TCP socket for Modbus TCP communication.
	*
	* Parameter:
	*   None
	*
	* Return:
	*   tHandle: TCP socket handle;
	*	  If successful, a TCP socket handle is returned.
	*     If not successful, IR_SOCKET_INVALID is returned.
	*
	* ********************************************************************************************* */
	EXPORTS tHandle __stdcall IR_MTCP_TCPNew(
		void
		);


	/* *********************************************************************************************
	* Function: IR_MTCP_TCPNew1
	*
	* Description:
	*	  Initialize a TCP socket bound to an NIC.
	*
	* Parameter:
	*   [in]ulNicIPAddr: IP address of a network card for binding. Network byte order. IP: 0xC0A80001 (=>192.168.0.1)
	*
	* Return:
	*   tHandle: Socket handle;
	*   If successful, a handle for the TCP socket is returned.
	*   If not successful, IR_SOCKET_INVALID is returned.
	*
	* ********************************************************************************************* */
	EXPORTS tHandle __stdcall IR_MTCP_TCPNew1(
		unsigned long ulNicIPAddr
		);


	/* *********************************************************************************************
	* Function: IR_MTCP_TCPClose
	*
	* Description:
	*	  Disconnect and Close a TCP socket.
	*
	* Parameter:
	*   [in]hSocketTCP: Socket handle
	*
	* Return:
	*   IR_E_OK: No error.
	*   IR_E_FUNC_ERR: Function error
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MTCP_TCPClose(
		tHandle hSocketTCP
		);


	/* *********************************************************************************************
	* Function: IR_MTCP_Connect
	*
	* Description:
	*	  Connect to the IR module with Modbus TCP.
	*
	* Parameter:
	*   [in]hSocketTCP: Socket handle
	*   [in]ulIPAddr: unsigned long value of the IP address of the IR module. IP: 192.168.0.1 => 0xC0A80001
	*   [in]NetID: Modbus NetID 0~247
	*   [in]timeoutMs: connection timeout (ms)
	*
	* Return:
	*   IR_E_OK: No error.
	*	IR_E_SOCKET_CONN_TOUT: connection timeout
	*   IR_E_FUNC_ERR: Function error
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MTCP_Connect(
		tHandle hSocketTCP,
		unsigned long ulIPAddr,
		unsigned char ucNetID,
		unsigned short timeoutMs
		);


	/* *********************************************************************************************
	* Function: IR_MTCP_GetModbusRecvTimeout
	*
	* Parameter:
	*   None
	*
	* Description:
	*	Get the receive timeout (ms) of Modbus TCP responses from the IR module.
	*   Default value = 200 ms
	*
	* Return:
	*   Receive timeout (ms) for the Modbus TCP response
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MTCP_GetModbusRecvTimeout(
		void
		);


	/* *********************************************************************************************
	* Function: IR_MTCP_SetModbusRecvTimeout
	*
	* Description:
	*   Set the receive timeout (ms) of Modbus TCP responses from the IR module.
	*   Default value = 200 ms.
	*
	* Parameter:
	*   [in]recvTout: Receive timeout for the Modbus TCP response after sending a Modbus TCP request.
	*
	* Return:
	*   None
	*
	* ********************************************************************************************* */
	EXPORTS void __stdcall IR_MTCP_SetModbusRecvTimeout(
		unsigned short recvTout
		);


	/* *********************************************************************************************
	* Function: IR_MTCP_GetBasicSetting
	*
	* Description:
	*   Get the basic settings from the IR remote module with Modbus TCP protocol.
	*
	* Parameter:
	*   [in]hSocketTCP:     TCP Socket handle
	*   [out]EBasicSetting: BasicSetParams structure variable for the bsaic settings of the IR learning
	*                      remote module with Modbus/TCP protocol.
	*
	* Return:
	*  IR_E_OK:  No Error
	*  IR_E_SEND_MBCMD_ERR:  Error in sending/receiving Modbus command.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MTCP_GetBasicSetting(
		tHandle hSocketTCP,
		/*[out]*/BasicSetParamsMTCP *pEBasicSettingMTCP
		);


	/* *********************************************************************************************
	* Function: IR_MTCP_SetBasicSetting
	*
	* Description:
	*   Set the basic settings to the IR MTCP remote module.
	*   Take care the times to call this function in programming because there are max 100000 times
	*   to write basic settings to the flash memory.
	*   After calling this function, reboot the IR module to take effect the settings.
	*
	* Parameter:
	*   [in]tHandle hSocketTCP,
	*   [in]EBasicSetting: BasicSetParams structure variable for the bsaic settings of the IR learning
	*                      remote module with Modbus/TCP protocol.
	*   [in]ucReboot:  Reset(reboot) the module after set the basic settings. 0=>nothing.  1=>write & reset, 2=>only write, no reset
	*
	* Return:
	*	IR_E_OK:  No Error
	*	IR_E_FUNC_ERR: Function Error (Parameter error in BasicSetParamsMTCP struct)
	*   IR_E_SEND_MBCMD_FLASH_ERR: Error in sending Modbus command to write data to Flash.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MTCP_SetBasicSetting(
		tHandle hSocketTCP,
		/*[in]*/BasicSetParamsMTCP EBasicSettingMTCP,
		unsigned char ucReboot
		);


	/* *********************************************************************************************
	* Function: IR_MTCP_RebootIRModule
	*
	* Description:
	*   Reboot IR Module with MBTCP protocol.
	*
	* Parameter:
	*   [in]hSocketTCP: TCP Socket handle
	*
	* Return:
	*	 IR_E_OK:  No Error
	*	 IR_E_SEND_MBCMD_ERR: Error in sending/receiving Modbus command.
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MTCP_RebootIRModule(
		tHandle hSocketTCP
		);


	/* *********************************************************************************************
	* Function: IR_MTCP_LrnModeSet
	*
	* Description:
	*   Enable/disable IR learning mode for IR module with MBTCP.
	*
	* Parameter:
	*   [in]hSocketTCP: TCP Socket handle
	*   [in]ucMode: Learning mode. 0 => learn off, 1=> learn on.
	*
	* Return:
	*	IR_E_OK: No errors.
	*	IR_E_SEND_MBCMD_ERR: Error in sending/receiving Modbus command.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MTCP_LrnModeSet(
		tHandle hSocketTCP,
		unsigned char ucMode
		);


	/* *********************************************************************************************
	* Function: IR_MTCP_IsIRLearnOK
	*
	* Description:
	*   Show the status of IR learnig by receiving the learning OK message once after learning
	*   an IR command is finished. (For IR MBTCP modules)
	*
	* Parameter:
	*   [in]hSocketTCP: TCP Socket handle
	*   [out]*pusIsOK: 0=>IR learning not OK, 1=>IR learning OK
	*                  The status 1 is back to 0 right away after the function was called. 
	*   [in]sTimeout: receive timeout: 0~32767 (ms)
	*
	* Return:
	*	IR_E_OK: No errors.
	*   IR_E_FUNC_ERR: Function erros.
	*	IR_E_SOCKET_RECV_TOUT: receive timeout
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MTCP_IsIRLearnOK(
		tHandle hSocketTCP,
		unsigned short *pusIsOK,
		int sTimeout
		);


	/* **************************************************************************************************
	* Function: IR_MTCP_GetIRCmdAfterLearn
	*
	* Description:
	*	Get one IR learning command data(without device name & IR cmd name) from the IR learning
	*   remote module to the buffer of the API library right away after learning an IR command successfully.
	*
	* Parameter:
	*   [in]hSocketTCP: TCP Socket handle
	*   [in]usIrCmdNumber: IR command number for allocating one IR learning command data
	*                      in the learning data buffer. (1 ~ 512, for IR-712-MTCP)
	*
	* Return:
	*   IR_E_OK:  No errors.
	*   IR_E_ILL_IRCMD_NO:  Illegal IR command number.
	*   IR_E_SEND_MBCMD_ERR: No response
	*
	*
	* ************************************************************************************************** */
	EXPORTS int __stdcall IR_MTCP_GetIRCmdAfterLearn(
		tHandle hSocketTCP,
		unsigned short usIrCmdNumber
		);


	/* *********************************************************************************************
	 * Function: IR_MTCP_EmitIrSignal
	 *
	 * Description:
	 *	 Request the IR module to emit an IR command stored in the flash memory of the IR module.
	 *   Emit IR signal with selected IR command number and IR output channels.
	 *
	 * Parameter:
	 *   [in]hSocketTCP: TCP Socket handle
	 *   [in]usIrCmdNumber: IR command number for allocating one IR learning command data
	 *                           in the learning data buffer
	 *   [in]ucIrOutputCh: IR output channels.  Use IR output ch1 & ch2 => value = 0x03 (0000 0011(binary))
	 *
	 * Return:
	 *   IR_E_OK: No errors.
	 *   IR_E_ILL_IRCMD_NO:  Illegal IR command number.
	 *   IR_E_SEND_MBCMD_ERR: Error in sending/receiving Modbus command.
	 *
	 * ********************************************************************************************* 
	 */
	EXPORTS int __stdcall IR_MTCP_EmitIrSignal(
		tHandle hSocketTCP,
		unsigned short usIrCmdNumber,
		unsigned char ucIrOutputCh
		);



	/* *********************************************************************************************
	* Function: IR_MTCP_RunCommand
	*
	* Description:
	*   Send an IR learning command data to the Modbus memory of the IR module and request IR module to emit that IR command.
	*
	* Parameter:
	*   [in]hSocketTCP: TCP Socket handle
	*   usIrCmdNumber[in]: IR command number for allocating one IR learning command data
	*                      in the learning data buffer. (1 ~ 512, for IR-712-MTCP)
	*   ucIrOutputCh[in]: IR output channels
	*
	*
	* Return:
	*   IR_E_OK: No errors.
	*   IR_E_IRCMD_DATA_EMPTY:  NO IR data exists for the specified IR cmd number in the API buffer.
	*   IR_E_ILL_IRCMD_NO:  Illegal IR command number.
	*   IR_E_LRNCMD_LEN_EXCEED_MAX:  The length of the IR cmd learning data exceeds the maximum.
	*   IR_E_SEND_MBCMD_ERR: Error in sending/receiving Modbus command.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MTCP_RunCommand(
		tHandle hSocketTCP,
		unsigned short usIrCmdNumber,
		unsigned char ucIrOutputCh
		);


	/* *********************************************************************************************
	* Function: IR_MTCP_DLOneIRDataToModule
	*
	* Description:
	*	 Download one IR cmd data (:= IR device name + IR cmd name + IR cmd learning data) from the
	*   API buffer to the IR Module.
	*
	* Parameter:
	*   [in]hSocketTCP: TCP Socket handle
	*   [in]usIrCmdNumber: IR command number for allocating one IR learning command data
	*                      in the learning data buffer
	*
	* Return:
	*   IR_E_OK: No errors.
	*   IR_E_ILL_IRCMD_NO: Illegal IR command number
	*   IR_E_SEND_MBCMD_ERR:  Error in sending/receiving Modbus command.
	*   IR_E_SEND_MBCMD_FLASH_ERR:  Error in sending/receiving Modbus command to write data to Flash.
	*   IR_E_LRNCMD_LEN_EXCEED_MAX:  The length of the IR cmd learning data exceeds the maximum.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MTCP_DLOneIRDataToModule(
		tHandle hSocketTCP,
		unsigned short usIrCmdNumber
		);


	/* *********************************************************************************************
	* Function: IR_MTCP_LDOneIRDataToAPI
	*
	* Description:
	*	 Load one IR data (IR device name + IR cmd name + IR learning) from an IR module to the buffer
	*   of API library.
	*
	* Parameter:
	*   [in]hSocketTCP: TCP Socket handle
	*   [in]usIrCmdNumber: IR command number. (1 ~ 512, for IR-712-MTCP)
	*
	* Return:
	*   IR_E_OK: No errors.
	*   IR_E_ILL_IRCMD_NO: Illegal IR command number
	*   IR_E_SEND_MBCMD_ERR:  Error in sending/receiving Modbus command.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MTCP_LDOneIRDataToAPI(
		tHandle hSocketTCP,
		unsigned short usIrCmdNumber
		);


	/* *********************************************************************************************
	* Function: IR_MTCP_ChkModuleName
	*
	* Description:
	*   Get model name of an IR remote module.
	*
	* Parameter:
	*   [in]hSocketTCP: TCP Socket handle
	*   [out]pwModuleIndex: Module index representing for specific module type(name),
	*         e.g. 0x21=>"IR-712-MTCP"
	*
	* Return:
	*   IR_E_OK:  No errors.
	*   IR_E_SEND_MBCMD_ERR:  Error in sending/receiving Modbus command.
	*   IR_E_MODNAME_INVALID:  Get invalid module index(name).
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MTCP_ChkModuleName(
		tHandle hSocketTCP,
		unsigned short *pwModuleIndex
		);


	/* *********************************************************************************************
	* Function: IR_MTCP_GetIRCmdSummary
	*
	* Description:
	*   Read IR command summary from the IR learning module. The IR command summary is also saved 
	*   to the buffer of the IR API Library.
	*
	* Parameter:
	*   [in]hSocketTCP: TCP Socket handle
	*   [in]usIrModuleIndex: Index of IR learning remote module
	*                              0x21: IR-712-MTCP
	*   [out]*dataArr: data array for IR command summary. Array length = 13
	*                     where dataArr[0]: IR device quantity (1 ~ 11)
	*                           dataArr[1]: Total quantity of IR commands ( 1 ~ 224 )
	*                           dataArr[2]: IR command quantity of device#1
	*                           dataArr[3]: IR command quantity of device#2
	*                             .....         .........................
	*                           dataArr[N+1]: IR command quantity of device#N , where N<=11
	*
	*   [in]usInElementCnt: Length of the dataArr.  Should be at least 13.
	*
	*
	* Return:
	*   IR_E_OK: No errors.
	*   IR_E_ILL_SUMMARY_LEN:  Illegal length of the array(dataArr) for IR command summary
	*	IR_E_DEVICE_QTY_ERR: The device quantity exceeds the maximum.
	*	IR_E_IRCMD_TOTAL_QTY_ERR:  The device quantity setting exceeds the maximum.
	*   IR_E_IRCMD_SUM_QTY_ERR:  The element "Total quantity of IR commands" in the IR cmd summary
	*                            exceeds the maximum.
	*   IR_E_ILL_MODULE_INDEX:  Illegal module index.
	*   IR_E_SEND_MBCMD_ERR:  Error in sending/receiving Modbus command.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MTCP_GetIRCmdSummary(
		tHandle hSocketTCP,
		unsigned short usIrModuleIndex,
		unsigned short *dataArr,
		unsigned short usInElementCnt
		);


	/* *********************************************************************************************
	* Function: IR_MTCP_SetIRCmdSummary
	*
	* Description:
	*   Write the IR command summary from the library buffer to the flash memory
	*   of the IR learning module.
	*   !Note: IR_SetIRCmdSummaryBuffer() should be called first.
	*
	* Parameter:
	*   [in]hSocketTCP: TCP Socket handle
	*   [in]usIrModuleIndex: Index value of IR learning remote module.  0x21: IR-712-MTCP
	*
	* Return:
	*	IR_E_OK: No errors.
	*   IR_E_DEVICE_QTY_ERR:  The device quantity exceeds the maximum.
	*	IR_E_IRCMD_TOTAL_QTY_ERR:  The device quantity setting exceeds the maximum.
	*	IR_E_IRCMD_SUM_QTY_ERR:  The element "Total quantity of IR commands" in the IR cmd summary
	*                            exceeds the maximum.
	*   IR_E_ILL_MODULE_INDEX:  Illegal module index.
	*   IR_E_SEND_MBCMD_ERR:  Error in sending/receiving Modbus command.
	*   IR_E_SEND_MBCMD_FLASH_ERR:  Error in sending/receiving Modbus command to write data to Flash.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MTCP_SetIRCmdSummary(
		tHandle hSocketTCP,
		unsigned short usIrModuleIndex
		);


	/* *********************************************************************************************
	* Function: IR_MTCP_SetIRCmdSummary1
	*
	* Description:
	*   Write IR command summary to the flash memory of the IR learning module.
	*
	* Parameter:
	*   [in]hSocketTCP: TCP Socket handle
	*   [in]usIrModuleIndex: Index of IR learning remote module
	*                              0x21: IR-712-MTCP
	*   [in]*dataArr: (Unsigned short) data array for IR command summary. Array length = 13
	*                     where dataArr[0]: IR device quantity (1 ~ 11)
	*                           dataArr[1]: Total quantity of IR commands ( 1 ~ 224 )
	*                           dataArr[2]: IR command quantity of device#1
	*                           dataArr[3]: IR command quantity of device#2
	*                             .....         .........................
	*                           dataArr[N+1]: IR command quantity of device#N , where N<=11
	*
	*   [in]usInElementCnt: Length of the dataArr. Should contain 13 elements exactly.
	*
	*
	* Return:
	*	IR_E_OK: No errors.
	*	IR_E_ILL_SUMMARY_LEN:  Illegal length of the array for IR command summary (should be 13)
	*	IR_E_FUNC_ERR:  Function error.
	*   IR_E_DEVICE_QTY_ERR:  The device quantity exceeds the maximum.
	*	IR_E_IRCMD_TOTAL_QTY_ERR:  The device quantity setting exceeds the maximum.
	*	IR_E_IRCMD_SUM_QTY_ERR:  The element "Total quantity of IR commands" in the IR cmd summary
	*                            exceeds the maximum.
	*   IR_E_ILL_MODULE_INDEX:  Illegal module index.
	*   IR_E_SEND_MBCMD_ERR:  Error in sending/receiving Modbus command.
	*   IR_E_SEND_MBCMD_FLASH_ERR:  Error in sending/receiving Modbus command to write data to Flash.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MTCP_SetIRCmdSummary1(
		tHandle hSocketTCP,
		unsigned short usIrModuleIndex,
		unsigned short *dataArr,
		unsigned short usInElementCnt
		);




	// **********************************************************
	//  ************   Modbus/UDP communication
	// **********************************************************

	/* *********************************************************************************************
	* Function: IR_MUDP_UDPNew
	*
	* Description:
	*	  Initailize a UDP socket for Modbus UDP.
	*
	* Parameter:
	*   [in]ulIPAddr: IP addresss of the IR module (UDP server). IP: 192.168.0.1 => 0xC0A80001
	*   [in]ucNetID: Modbus Net ID: 1 ~247
	*
	* Return:
	*   tHandle: UDP socket handle;
	*	  If successful, a UDP socket handle is returned.
	*     If not successful, IR_SOCKET_INVALID is returned.
	*
	* ********************************************************************************************* */
	EXPORTS tHandle __stdcall IR_MUDP_UDPNew(
		unsigned long ulIPAddr,
		unsigned char ucNetID
		);


	/* *********************************************************************************************
	* Function: IR_MUDP_UDPNew1
	*
	* Description:
	*	  Initailize a UDP socket which binds to a specific network interface(e.g. Ethernet card, Wi-Fi).
	*
	* Parameter:
	*   [in]ulNicIPAddr: IP address of a network card for binding. IP: 192.168.0.1 => 0xC0A80001
	*   [in]ulIPAddr: IP addresss of the IR Module(UDP server). IP: 192.168.0.2 => 0xC0A80002
	*	[in]ucNetID: Modbus Net ID: 1 ~247
	*
	* Return:
	*   tHandle: Socket handle;
	*   If successful, a handle for the UDP socket is returned.
	*   If not successful, IR_E_SOCKET_INVALID is returned.
	*
	* ********************************************************************************************* */
	EXPORTS tHandle __stdcall IR_MUDP_UDPNew1(
		unsigned long ulNicIPAddr,
		unsigned long ulIPAddr,
		unsigned char ucNetID
		);


	/* *********************************************************************************************
	* Function: IR_MUDP_UDPClose
	*
	* Description:
	*	  Close and release a UDP socket.
	*
	* Parameter:
	*   [in]hSocketUDP: Modbus UDP Socket handle
	*
	* Return:
	*   IR_E_OK: No error.
	*   IR_E_FUNC_ERR: Function error
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MUDP_UDPClose(
		tHandle hSocketUDP
		);


	/* *********************************************************************************************
	* Function: IR_MUDP_GetBasicSetting
	*
	* Description:
	*   Get the basic settings from the IR remote module with Modbus UDP protocol.
	*
	* Parameter:
	*   [in]hSocketUDP: Modbus UDP socket handle
	*   [out]pEBasicSettingMUDP: BasicSetParamsMTCP structure variable for the bsaic settings of the IR learning
	*					         remote module with Modbus UDP protocol.
	*
	* Return:
	*  IR_E_OK:  No Error
	*  IR_E_SEND_MBCMD_ERR:  Error in sending/receiving Modbus command.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MUDP_GetBasicSetting(
		tHandle hSocketUDP,
		/*[out]*/BasicSetParamsMTCP *pEBasicSettingMUDP
		);


	/* *********************************************************************************************
	* Function: IR_MUDP_SetBasicSetting
	*
	* Description:
	*   Set the basic settings to the IR MTCP remote module with Modbus/UDP protocol.
	*   Take care the times to call this function in programming because there are max 100000 times
	*   to write basic settings to the flash memory.
	*   Them reset the module immediately.
	*
	* Parameter:
	*   [in]tHandle hSocketUDP,
	*   [in]EBasicSettingMUDP: BasicSetParams structure variable for the bsaic settings of the IR learning
	*                      remote module with Modbus UDP protocol.
	*   [in]ucReboot:  Reset(reboot) the module after set the basic settings. 0=>nothing.  1=>write & reset, 2=>only write, no reset
	*
	* Return:
	*	IR_E_OK:  No Error
	*	IR_E_FUNC_ERR: Function Error (Parameter error in BasicSetParamsMTCP struct)
	*   IR_E_SEND_MBCMD_FLASH_ERR: Error in sending Modbus command to write data to Flash.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MUDP_SetBasicSetting(
		tHandle hSocketUDP,
		/*[in]*/BasicSetParamsMTCP EBasicSettingMUDP,
		unsigned char ucReboot
		);


	/* *********************************************************************************************
	* Function: IR_MUDP_RebootIRModule
	*
	* Description:
	*   Reboot IR Module with MBUDP protocol.
	*
	* Parameter:
	*   [in]hSocketUDP: UDP Socket handle
	*
	* Return:
	*   IR_E_OK:  No Error
	*   IR_E_SEND_MBCMD_ERR: Error in sending/receiving Modbus command.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MUDP_RebootIRModule(
		tHandle hSocketUDP
		);


	/* *********************************************************************************************
	* Function: IR_MUDP_GetModbusRecvTimeout
	*
	* Parameter:
	*   None
	*
	* Description:
	*	Get the receive timeout (ms) of Modbus UDP responses from the IR module.
	*   Default value = 200 ms
	*
	* Return:
	*   Timeout (ms) for receiving Modbus response. Default value = 200 ms.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MUDP_GetModbusRecvTimeout(
		void
		);


	/* *********************************************************************************************
	* Function: IR_MUDP_SetModbusRecvTimeout
	*
	* Description:
	*   Set the receive timeout (ms) of Modbus UDP responses from the IR module.
	*   Default value = 200 ms.
	*
	* Parameter:
	*   [in]recvTout: Receive timeout for the Modbus UDP response after sending a Modbus UDP request.
	*
	* Return:
	*   None
	*
	* ********************************************************************************************* */
	EXPORTS void __stdcall IR_MUDP_SetModbusRecvTimeout(
		unsigned short recvTout
		);


	/* *********************************************************************************************
	* Function: IR_MUDP_LrnModeSet
	*
	* Description:
	*   Enable/disable IR learning mode for IR module with MBUDP.
	*
	* Parameter:
	*   [in]hSocketUDP: UDP Socket handle
	*   [in]ucMode: Learning mode. 0 => learn off, 1=> learn on.
	*
	* Return:
	*	IR_E_OK: No errors.
	*	IR_E_SEND_MBCMD_ERR: Error in sending/receiving Modbus command.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MUDP_LrnModeSet(
		tHandle hSocketUDP,
		unsigned char ucMode
		);


	/* *********************************************************************************************
	* Function: IR_MUDP_IsIRLearnOK
	*
	* Description:
	*   Show the status of IR learnig by receiving the learning OK message once after learning
	*   an IR command is finished. (For IR modules with MBUDP)
	*
	* Parameter:
	*   [in]hSocketUDP: UDP Socket handle
	*   [out]*pusIsOK: 0=>IR learning not OK, 1=>IR learning OK
	*   [in]sTimeout: receive timeout: 0~32767 (ms)
	*
	* Return:
	*	IR_E_OK: No errors.
	*   IR_E_FUNC_ERR: Function erros.
	*	IR_E_SOCKET_RECV_TOUT: receive timeout
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MUDP_IsIRLearnOK(
		tHandle hSocketUDP,
		unsigned short *pusIsOK,
		int sTimeout
		);


	/* *********************************************************************************************
	* Function: IR_MUDP_EmitIrSignal
	*
	* Description:
	*	Request the IR module to emit an IR command stored in the flash memory of the IR module.
	*   Emit IR signal by selected IR command number and IR output channels.
	*
	* Parameter:
	*   [in]hSocketUDP: UDP Socket handle
	*   [in]usIrCmdNumber: IR command number for allocating one IR learning command data
	*                      in the learning data buffer. (1 ~ 512, for IR-712-MTCP)
	*   [in]ucIrOutputCh: IR output channels.  Use IR output ch1 & ch2 => value = 0x03 (0000 0011(binary))
	*
	* Return:
	*   IR_E_OK: No errors.
	*   IR_E_ILL_IRCMD_NO:  Illegal IR command number.
	*   IR_E_SEND_MBCMD_ERR: Error in sending/receiving Modbus command.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MUDP_EmitIrSignal(
		tHandle hSocketUDP,
		unsigned short usIrCmdNumber,
		unsigned char ucIrOutputCh
		);


	/* *********************************************************************************************
	* Function: IR_MUDP_RunCommand
	*
	* Description:
	*   Send an IR command learning data to the IR module and emit that IR command.
	*
	* Parameter:
	*   [in]hSocketUDP: UDP Socket handle
	*   usIrCmdNumber[in]: IR command number for allocating one IR learning command data
	*                      in the learning data buffer
	*   ucIrOutputCh[in]: IR output channels
	*
	*
	* Return:
	*   IR_E_OK: No errors.
	*   IR_E_IRCMD_DATA_EMPTY:  NO IR data exists for the specified IR cmd number in the API buffer.
	*   IR_E_ILL_IRCMD_NO:  Illegal IR command number.
	*   IR_E_LRNCMD_LEN_EXCEED_MAX:  The length of the IR cmd learning data exceeds the maximum.
	*   IR_E_SEND_MBCMD_ERR: Error in sending/receiving Modbus command.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MUDP_RunCommand(
		tHandle hSocketUDP,
		unsigned short usIrCmdNumber,
		unsigned char ucIrOutputCh
		);


	/* **************************************************************************************************
	* Function: IR_MUDP_GetIRCmdAfterLearn
	*
	* Description:
	*	 Get one IR learning command data(without device name & IR cmd name) from the IR learning
	*   remote module to the buffer of the API library right after learning an IR command successfully.
	*
	* Parameter:
	*   [in]hSocketUDP: UDP Socket handle
	*   [in]usIrCmdNumber: IR command number for allocating one IR learning command data
	*                      in the learning data buffer
	*
	* Return:
	*   IR_E_OK:  No errors.
	*   IR_E_ILL_IRCMD_NO:  Illegal IR command number.
	*   IR_E_SEND_MBCMD_ERR: No response
	*
	*
	* ************************************************************************************************** */
	EXPORTS int __stdcall IR_MUDP_GetIRCmdAfterLearn(
		tHandle hSocketUDP,
		unsigned short usIrCmdNumber
		);


	/* *********************************************************************************************
	* Function: IR_MUDP_DLOneIRDataToModule
	*
	* Description:
	*	 Download one IR cmd data (:= IR device name + IR cmd name + IR cmd learning data) from the
	*   API buffer to the IR Module.
	*
	* Parameter:
	*   [in]hSocketUDP: UDP Socket handle
	*   [in]usIrCmdNumber: IR command number for allocating one IR learning command data
	*                      in the learning data buffer
	*
	* Return:
	*   IR_E_OK: No errors.
	*   IR_E_ILL_IRCMD_NO: Illegal IR command number
	*   IR_E_SEND_MBCMD_ERR:  Error in sending/receiving Modbus command.
	*   IR_E_SEND_MBCMD_FLASH_ERR:  Error in sending/receiving Modbus command to write data to Flash.
	*   IR_E_LRNCMD_LEN_EXCEED_MAX:  The length of the IR cmd learning data exceeds the maximum.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MUDP_DLOneIRDataToModule(
		tHandle hSocketUDP,
		unsigned short usIrCmdNumber
		);


	/* *********************************************************************************************
	* Function: IR_MUDP_LDOneIRDataToAPI
	*
	* Description:
	*	 Load one IR data (IR device name + IR cmd name + IR learning) from an IR module to the buffer
	*   of API library.
	*
	* Parameter:
	*   [in]hSocketUDP: UDP Socket handle
	*   [in]usIrCmdNumber: IR command number for allocating one IR learning command data
	*                      in the learning data buffer
	*
	* Return:
	*   IR_E_OK: No errors.
	*   IR_E_ILL_IRCMD_NO: Illegal IR command number
	*   IR_E_SEND_MBCMD_ERR:  Error in sending/receiving Modbus command.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MUDP_LDOneIRDataToAPI(
		tHandle hSocketUDP,
		unsigned short usIrCmdNumber
		);


	/* *********************************************************************************************
	* Function: IR_MUDP_ChkModuleName
	*
	* Description:
	*   Get model name of the IR remote module.
	*
	* Parameter:
	*   [in]hSocketUDP: UDP Socket handle
	*   [out]pwModuleIndex: Module index representing for specific module type(name),
	*         e.g. 0x21=>"IR-712-MTCP"
	*
	* Return:
	*   IR_E_OK:  No errors.
	*   IR_E_SEND_MBCMD_ERR:  Error in sending/receiving Modbus command.
	*   IR_E_MODNAME_INVALID:  Get invalid module index(name).
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MUDP_ChkModuleName(
		tHandle hSocketUDP,
		unsigned short *pwModuleIndex
		);


	/* *********************************************************************************************
	* Function: IR_MUDP_GetIRCmdSummary
	*
	* Description:
	*   Read IR command summary from the IR learning module.
	*
	* Parameter:
	*   [in]hSocketUDP: UDP Socket handle
	*   [in]usIrModuleIndex: Index of IR learning remote module
	*                              0x21: IR-712-MTCP
	*   [out]*dataArr: data array for IR command summary. Array length = 13
	*                     where dataArr[0]: IR device quantity (1 ~ 11)
	*                           dataArr[1]: Total quantity of IR commands ( 1 ~ 224 )
	*                           dataArr[2]: IR command quantity of device#1
	*                           dataArr[3]: IR command quantity of device#2
	*                             .....         .........................
	*                           dataArr[N+1]: IR command quantity of device#N , where N<=11
	*
	*   [in]usInElementCnt: Length of the dataArr.  Should be at least 13.
	*
	*
	* Return:
	*   IR_E_OK: No errors.
	*   IR_E_ILL_SUMMARY_LEN:  Illegal length of the array(dataArr) for IR command summary
	*	IR_E_DEVICE_QTY_ERR: The device quantity exceeds the maximum.
	*	IR_E_IRCMD_TOTAL_QTY_ERR:  The device quantity setting exceeds the maximum.
	*   IR_E_IRCMD_SUM_QTY_ERR:  The element "Total quantity of IR commands" in the IR cmd summary
	*                            exceeds the maximum.
	*   IR_E_ILL_MODULE_INDEX:  Illegal module index.
	*   IR_E_SEND_MBCMD_ERR:  Error in sending/receiving Modbus command.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MUDP_GetIRCmdSummary(
		tHandle hSocketUDP,
		unsigned short usIrModuleIndex,
		unsigned short *dataArr,
		unsigned short usInElementCnt
		);


	/* *********************************************************************************************
	* Function: IR_MUDP_SetIRCmdSummary
	*
	* Description:
	*   Write the IR command summary from the library buffer to the flash memory
	*   of the IR learning module.
	*   !Note: IR_SetIRCmdSummaryBuffer() should be called first.
	*
	* Parameter:
	*   [in]hSocketTCP: UDP Socket handle
	*   [in]usIrModuleIndex: Index value of IR learning remote module.  0x21: IR-712-MTCP
	*
	* Return:
	*	IR_E_OK: No errors.
	*   IR_E_DEVICE_QTY_ERR:  The device quantity exceeds the maximum.
	*	IR_E_IRCMD_TOTAL_QTY_ERR:  The device quantity setting exceeds the maximum.
	*	IR_E_IRCMD_SUM_QTY_ERR:  The element "Total quantity of IR commands" in the IR cmd summary
	*                            exceeds the maximum.
	*   IR_E_ILL_MODULE_INDEX:  Illegal module index.
	*   IR_E_SEND_MBCMD_ERR:  Error in sending/receiving Modbus command.
	*   IR_E_SEND_MBCMD_FLASH_ERR:  Error in sending/receiving Modbus command to write data to Flash.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MUDP_SetIRCmdSummary(
		tHandle hSocketUDP,
		unsigned short usIrModuleIndex
		);


	/* *********************************************************************************************
	* Function: IR_MUDP_SetIRCmdSummary1
	*
	* Description:
	*   Write IR command summary to the flash memory of the IR learning module.
	*
	* Parameter:
	*   [in]hSocketUDP: UDP Socket handle
	*   [in]usIrModuleIndex: Index of IR learning remote module
	*                              0x21: IR-712-MTCP
	*   [in]*dataArr: (Unsigned short) data array for IR command summary. Array length = 13
	*                     where dataArr[0]: IR device quantity (1 ~ 11)
	*                           dataArr[1]: Total quantity of IR commands ( 1 ~ 224 )
	*                           dataArr[2]: IR command quantity of device#1
	*                           dataArr[3]: IR command quantity of device#2
	*                             .....         .........................
	*                           dataArr[N+1]: IR command quantity of device#N , where N<=11
	*
	*   [in]usInElementCnt: Length of the dataArr. Should contain 13 elements exactly.
	*
	*
	* Return:
	*	IR_E_OK: No errors.
	*	IR_E_ILL_SUMMARY_LEN:  Illegal length of the array for IR command summary (should be 13)
	*	IR_E_FUNC_ERR:  Function error.
	*   IR_E_DEVICE_QTY_ERR:  The device quantity exceeds the maximum.
	*	IR_E_IRCMD_TOTAL_QTY_ERR:  The device quantity setting exceeds the maximum.
	*	IR_E_IRCMD_SUM_QTY_ERR:  The element "Total quantity of IR commands" in the IR cmd summary
	*                            exceeds the maximum.
	*   IR_E_ILL_MODULE_INDEX:  Illegal module index.
	*   IR_E_SEND_MBCMD_ERR:  Error in sending/receiving Modbus command.
	*   IR_E_SEND_MBCMD_FLASH_ERR:  Error in sending/receiving Modbus command to write data to Flash.
	*
	* ********************************************************************************************* */
	EXPORTS int __stdcall IR_MUDP_SetIRCmdSummary1(
		tHandle hSocketUDP,
		unsigned short usIrModuleIndex,
		unsigned short *dataArr,
		unsigned short usInElementCnt
		);



#ifdef __cplusplus
}
#endif

#endif   // End of #define __IRLRNAPI_H__

