﻿//
// Modbus RTU C#.NET sample for IR API Library.
// This demo application is a Visual Studi 2019 C# WPF project for IR-210/IR-712A.
// 
// IR API WPF C# Sample
// 
// Release Note:
//   [v2.2.0.0], 2022-June-02
//


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Threading;
using IRAPI_Ns;


namespace IrAPI_CSharpWPFDemo
{
    /// <summary>
    /// Interaction logic for IRMRTUDemoWin0.xaml
    /// </summary>
    public partial class IRMBRTUDemoWin0 : Window
    {
        public IRMBRTUDemoWin0()
        {
            InitializeComponent();
        }



        private Byte _comPortNo = 6;          // COM port number
        private UInt32 _baudRate = 115200;    // baud rate (bps)
        private Byte _baudRateIndex = 10;     // baudrate index [6] ~ [10]
        private Byte _databits = 8;           // data bits
        private Byte _parity = 0;             // parity
        private Byte _stopbitsIndex = 0;      // stopbits index 0: databits == 1, 2: databits == 2
        private Byte _mbID = 1;               // Modbus Net ID

        
        private Byte[] IROutputChArr = new Byte[7] { 0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x3F };    // Combinations of IR output channels.
        private String[] deviceNameArr = new string[11];                    // Strings array for IR device names.
        private String[] cmdNameArr = new String[512];                      // Strings array for IR cmd names.


        private int[] irCmdQtyInDevice = new int[3] { 1, 2, 3 };
        private int IrDeviceTotalQty = 3,
                    IrCmdCurrTotalCmd = 6;

        private int[] IrCmdSummaryArr = new int[13];

        private Thread thDwLDIR;    // Thread for downloading IR cmds to the IR module and updating IR cmd numbers in the Textbox "txtIRCmdQtyCurr".
        private Thread thLDIR;      // Thread for loading IR cmds from the IR module and updating IR cmd numbers in the Textbox "txtIRCmdQtyCurr".



        private void IRMBRTUDemoWin0_Loaded(object sender, RoutedEventArgs e)
        {
            this.Height = 600;

            // Com port combobox
            cboCOMPort.Items.Clear();
            for (int i = 0; i < 32; i++)
            {
                cboCOMPort.Items.Add("COM" + (i + 1).ToString());
            }
            cboCOMPort.SelectedIndex = 5;


            // Baud rate combobox
            cboBaudRate.Items.Clear();
            cboBaudRate.Items.Add("9600");
            cboBaudRate.Items.Add("19200");
            cboBaudRate.Items.Add("38400");
            cboBaudRate.Items.Add("57600");
            cboBaudRate.Items.Add("115200");
            cboBaudRate.SelectedIndex = 4;

            cboBaudRate_GS.Items.Add("9600");
            cboBaudRate_GS.Items.Add("19200");
            cboBaudRate_GS.Items.Add("38400");
            cboBaudRate_GS.Items.Add("57600");
            cboBaudRate_GS.Items.Add("115200");
            cboBaudRate_GS.SelectedIndex = 4;

            // Data bits combobox
            cboDataBits.Items.Clear();
            cboDataBits.Items.Add("8");
            cboDataBits.SelectedIndex = 0;

            cboDataBits_GS.Items.Clear();
            cboDataBits_GS.Items.Add("8");
            cboDataBits_GS.SelectedIndex = 0;



            // Parity combobox
            cboParity.Items.Clear();
            cboParity.Items.Add("None");
            cboParity.Items.Add("Odd");
            cboParity.Items.Add("Even");
            cboParity.SelectedIndex = 0;

            cboParity_GS.Items.Clear();
            cboParity_GS.Items.Add("None");
            cboParity_GS.Items.Add("Odd");
            cboParity_GS.Items.Add("Even");
            cboParity_GS.SelectedIndex = 0;



            // Stop bits combobox
            cboStopBits.Items.Clear();
            cboStopBits.Items.Add("1");
            cboStopBits.Items.Add("2");
            cboStopBits.SelectedIndex = 0;

            cboStopBits_GS.Items.Clear();
            cboStopBits_GS.Items.Add("1");
            cboStopBits_GS.Items.Add("2");
            cboStopBits_GS.SelectedIndex = 0;



            // Modbus Net ID combobox
            cboMBNetID2.Items.Clear();
            cboMBNetID2.Items.Clear();
            for (int j = 1; j < 248; j++)
            {
                cboMBNetID2.Items.Add(j.ToString());
                cboMBNetID2_GS.Items.Add(j.ToString());
            }
            cboMBNetID2.SelectedIndex = 0;
            cboMBNetID2_GS.SelectedIndex = 0;


           

            // Init. the combobox of IR command number
            cboIRCmdNo.Items.Clear();
            cboIRCmdNo2.Items.Clear();
            for (int i = 1; i < 513; i++)
            {
                cboIRCmdNo.Items.Add(i.ToString());
                cboIRCmdNo2.Items.Add(i.ToString());
            }
            cboIRCmdNo.SelectedIndex = 0;
            cboIRCmdNo2.SelectedIndex = 0;

            // Init. the combobox of IR output channel
            cboIROutputCh.Items.Clear();
            for (int i = 1; i < 7; i++ )
            {
                cboIROutputCh.Items.Add(i.ToString());
            }
            cboIROutputCh.Items.Add("1+2+3+4+5+6");
            cboIROutputCh.SelectedIndex = 0;

            cboIROutputCh2.Items.Clear();
            cboIROutputCh2.Items.Add("1");
            cboIROutputCh2.Items.Add("2");
            cboIROutputCh2.Items.Add("1 + 2");
            cboIROutputCh2.SelectedIndex = 0;


            // Init. Device name comboboxe
            deviceNameArr[0] = "TV";                          // IR device #1
            deviceNameArr[1] = "Audio";                       // IR device #2
            deviceNameArr[2] = "A/C";           //"空調";     // IR device #3

            cboDeviceName.Items.Clear();
            for (int i = 0; i < deviceNameArr.Length; i++)
            {
                cboDeviceName.Items.Add((i + 1).ToString() + "_" + deviceNameArr[i]);
            }
            cboDeviceName.SelectedIndex = 0;


            // Init. IR command name combobox
            cmdNameArr[0] = "Power Toggle";     //"開關";     // IR cmd #1
            cmdNameArr[1] = "Vol+";                           // IR cmd #2
            cmdNameArr[2] = "Vol-";                           // IR cmd #3
            cmdNameArr[3] = "Power Toggle";     //"開關";     // IR cmd #4
            cmdNameArr[4] = "Temperature+";     //"溫度+";    // IR cmd #5
            cmdNameArr[5] = "Wind Direction";   //"風向";     // IR cmd #6

            cboIrCmdNameArr.Items.Clear();
            for (int i = 0; i < cmdNameArr.Length; i++)
            {
                cboIrCmdNameArr.Items.Add((i + 1).ToString() + "_" + cmdNameArr[i]);
            }
            cboIrCmdNameArr.SelectedIndex = 0;


            // Init. the IR command summary data in the IR Cmd Data buffer in the IR API library.
            int rtn = 0;
            ushort moduleIndex = 0x21;          // Module index. 0x21: IR-712-MTCP
            ushort[] dataArr = new ushort[13];  // Array for IR command summary.
            IrCmdSummaryArr[0] = dataArr[0] = 3;                     // IR device quantity (#1, #2, #3)
            IrCmdSummaryArr[1] = dataArr[1] = 6;                     // Total IR command quantity
            IrCmdSummaryArr[2] = dataArr[2] = 1;                     // IR command quantity for IR device #1
            IrCmdSummaryArr[3] = dataArr[3] = 2;                     // IR command quantity for IR device #2
            IrCmdSummaryArr[4] = dataArr[4] = 3;                     // IR command quantity for IR device #3

            rtn = IR_API_Class.IR_SetIRCmdSummaryBuffer(moduleIndex, dataArr, (ushort)dataArr.Length);  // Set IR cmd summary to buffer.



            // Init. the IR device name & command name data in the IR Cmd Data buffer in the IR API library.
            Byte[] IrDevNameArr = new Byte[100];
            Byte[] IrCmdNameArr = new Byte[100];

            for (int i = 0; i < dataArr[0]; i++)
            {
                switch (i)
                {
                    case 0: // for IR device #1
                        for (int j = 0; j < dataArr[2]; j++)
                        {
                            Byte[] IrDevNameArrTemp = Encoding.UTF8.GetBytes(deviceNameArr[i]);
                            Byte[] IrCmdNameArrTemp = Encoding.UTF8.GetBytes(cmdNameArr[j]);
                            rtn = IR_API_Class.IR_SetIrDevCmdNameInBuffer(
                                    (UInt16)(j + 1),
                                    IrDevNameArrTemp, (UInt16)IrDevNameArrTemp.Length,
                                    IrCmdNameArrTemp, (UInt16)IrCmdNameArrTemp.Length
                                );
                        }
                        break;
                    case 1: // for IR device #2
                        for (int j = 0; j < dataArr[3]; j++)
                        {
                            Byte[] IrDevNameArrTemp = Encoding.UTF8.GetBytes(deviceNameArr[i]);
                            Byte[] IrCmdNameArrTemp = Encoding.UTF8.GetBytes(cmdNameArr[j + dataArr[2]]);
                            rtn = IR_API_Class.IR_SetIrDevCmdNameInBuffer(
                                    (UInt16)(j + dataArr[2] + 1),
                                    IrDevNameArrTemp, (UInt16)IrDevNameArrTemp.Length,
                                    IrCmdNameArrTemp, (UInt16)IrCmdNameArrTemp.Length
                                );
                        }
                        break;
                    case 2: // for IR device #3
                        for (int j = 0; j < dataArr[4]; j++)
                        {
                            Byte[] IrDevNameArrTemp = Encoding.UTF8.GetBytes(deviceNameArr[i]);
                            Byte[] IrCmdNameArrTemp = Encoding.UTF8.GetBytes(cmdNameArr[j + dataArr[2] + dataArr[3]]);
                            rtn = IR_API_Class.IR_SetIrDevCmdNameInBuffer(
                                    (UInt16)(j + dataArr[2] + dataArr[3] + 1),
                                    IrDevNameArrTemp, (UInt16)IrDevNameArrTemp.Length,
                                    IrCmdNameArrTemp, (UInt16)IrCmdNameArrTemp.Length
                                );
                        }
                        break;
                    default:
                        break;
                }

            }


            // Init rectangle
            Rect1.Fill = new SolidColorBrush(System.Windows.Media.Colors.White);


        }




        private void btnGetAPIVer_Click(object sender, RoutedEventArgs e)
        {
            UInt32 verValue = IR_API_Class.IR_GetAPIVersion();
            String major = ((verValue >> 24) & 0xFF).ToString(),
                   minor = ((verValue >> 16) & 0xFF).ToString(),
                   revision = ((verValue >> 8) & 0xFF).ToString(),
                   buildNo = (verValue & 0xFF).ToString();

            txtAPIVer.Text = major + "." + minor + "." + revision + "." + buildNo;
        }



        private void btnGetAPIDate_Click(object sender, RoutedEventArgs e)
        {
            UInt32 dateValue = IR_API_Class.IR_GetAPIDate();
            String APIDate = (dateValue >> 16).ToString() + "-" + ((dateValue >> 8) & 0xFF).ToString() + "-" + (dateValue & 0xFF).ToString();
            txtAPIDate.Text = APIDate;
        }


        private void btnGetComPortStatus_Click(object sender, RoutedEventArgs e)
        {
            int rtn = IR_API_Class.IR_GetComPortStatus((Byte)(cboCOMPort.SelectedIndex + 1));

            if (rtn == IR_API_Class.IR_E_COM_STATUS_AVAIL)
            {
                txtGetComPortStatus.Text = "Available";
            }
            else
            {
                txtGetComPortStatus.Text = "Error";
            }
        }



        private void btnOpenCOM_Click(object sender, RoutedEventArgs e)
        {

            Int32 rtn;

            if (cboCOMPort.SelectedIndex < 0)
            {
                MessageBox.Show("No COM Port");
                return;
            }

            // Get COM port number
            _comPortNo = (Byte)(cboCOMPort.SelectedIndex + 1);

            // Get buad rate
            switch (cboBaudRate.SelectedIndex)
            {
                case 0:  // 9600 bps
                    _baudRate = 9600;
                    _baudRateIndex = 6;
                    break;

                case 1:  // 19200 bps
                    _baudRate = 19200;
                    _baudRateIndex = 7;
                    break;

                case 2:  // 38400 bps
                    _baudRate = 38400;
                    _baudRateIndex = 8;
                    break;

                case 3:  // 57600 bps
                    _baudRate = 57600;
                    _baudRateIndex = 9;
                    break;

                case 4:  // 115200 bps
                    _baudRate = 115200;
                    _baudRateIndex = 10;
                    break;
                default:
                    _baudRate = 115200;
                    _baudRateIndex = 10;
                    break;
            }

            // Get Data bits
            switch (cboDataBits.SelectedIndex)
            {
                case 0:
                    _databits = (Byte)8;
                    break;
                default:
                    _databits = (Byte)8;
                    break;
            }


            // Get parity
            switch (cboParity.SelectedIndex)
            {
                case 0:  // None
                    _parity = (Byte)0;
                    break;
                case 1:  // Odd
                    _parity = (Byte)1;
                    break;
                case 2:  // Even
                    _parity = (Byte)2;
                    break;
                default:
                    _parity = (Byte)0;
                    break;
            }

            // Get Stip bits index
            switch (cboStopBits.SelectedIndex)
            {
                case 0:
                    _stopbitsIndex = (Byte)1;   // stop bits = 1;
                    break;
                case 1:
                    _stopbitsIndex = (Byte)2;   // stop bits = 2
                    break;
                default:
                    _stopbitsIndex = (Byte)1;
                    break;
            }

            // Get Modbus Net ID
            _mbID = Byte.Parse(cboMBNetID2.Text);

            rtn = IR_API_Class.IR_OpenCom(_comPortNo, _baudRateIndex, _databits, _parity, _stopbitsIndex);
            txtOpenCOMRtn.Text = rtn.ToString();  // Show the return value of the UartOpenCom(). 

            if (rtn > 0)
            {   // Error
                MessageBox.Show("Open COM error. Error code = " + rtn.ToString(), "Error");
                btnOpenCOM.IsEnabled = true;


            }
            else
            {   // Open COM OK
                btnOpenCOM.IsEnabled = false;
            }

        }




        private void btnCloseCOM_Click(object sender, RoutedEventArgs e)
        {
            Int32 rtn = IR_API_Class.IR_CloseCom(_comPortNo);
            txtCloseCOMRtn.Text = rtn.ToString();
            btnOpenCOM.IsEnabled = true;

        }



        private void btnChkModuleName_Click(object sender, RoutedEventArgs e)
        {
            UInt16 moduleIndex = 0;
            _mbID = Byte.Parse(cboMBNetID2.Text);

            // Get the module name of the IR module.
            Int32 rtn = IR_API_Class.IR_ChkModuleName(_comPortNo, _mbID, ref moduleIndex);
            txtOpenComChkRtn.Text = rtn.ToString();
            if (moduleIndex == 0x01)
            {
                txtModuleIndex.Text = "IR-210";
            }
            else if (moduleIndex == 0x11)
            {
                txtModuleIndex.Text = "IR-712A";
            }
            else
            {
                txtModuleIndex.Text = "";
            }
            
        }






































        private void btnGetInterfaces_Click(object sender, RoutedEventArgs e)
        {
            int iNICCount = 0;
            IR_API_Class.IR_ChkNIC(ref iNICCount);
            MessageBox.Show("Number of NICs = " + iNICCount.ToString(), "Check NIC");

            // Show NIC description
            byte[] description = new byte[133];
            int actualLen = 0;
            UInt32 IPAddr = 0;
            Byte NB0, NB1, NB2, NB3;  // Network address:= NB0.NB1.NB2.NB3
            String showStr;

            for (int i = 0; i < iNICCount; i++)
            {
                IR_API_Class.IR_GetNICInfo((byte)i, description, description.Length, ref actualLen, ref IPAddr);
                NB0 = (Byte)(IPAddr >> 24);
                NB1 = (Byte)(IPAddr >> 16);
                NB2 = (Byte)(IPAddr >> 8);
                NB3 = (Byte)IPAddr;
                showStr = string.Format("NIC IPAddr = 0x" + IPAddr.ToString("X") + " := "
                        + NB0.ToString() + "." + NB1.ToString() + "." + NB2.ToString() + "." + NB3.ToString()
                        + "\nNIC Description = {0}", Encoding.ASCII.GetString(description));
                MessageBox.Show(showStr, "Check NIC Result");
            }

        }



        




        



        private void btnMRTUGetBasicSettings_Click(object sender, RoutedEventArgs e)
        {
            int rtn = 0;
            IR_API_Class.BasicSetParams mRtuBasicSetStruct = new IR_API_Class.BasicSetParams();

            // Get basic settings.
            rtn = IR_API_Class.IR_GetBasicSetting(_comPortNo, _mbID, ref mRtuBasicSetStruct);

            txtGetBasicSet.Text = rtn.ToString();

            if (rtn == IR_API_Class.IR_E_OK)
            {

                // Get baud rate
                switch(mRtuBasicSetStruct.ucBaudrateIndex)
                {
                    case 6: // 9600 bps
                        cboBaudRate_GS.SelectedIndex = 0;
                        break;
                    case 7: // 19200 bps
                        cboBaudRate_GS.SelectedIndex = 1;
                        break;
                    case 8: // 38400 bps
                        cboBaudRate_GS.SelectedIndex = 2;
                        break;
                    case 9: // 57600 bps
                        cboBaudRate_GS.SelectedIndex = 3;
                        break;
                    case 10: // 115200 bps
                        cboBaudRate_GS.SelectedIndex = 4;
                        break;
                    default:
                        break;
                }


                // Get databits
                switch(mRtuBasicSetStruct.ucData)
                {
                    case 8: // 8 databits
                        cboDataBits_GS.SelectedIndex = 0;
                        break;
                    default:
                        break;
                }
                

                // Get Parity
                switch(mRtuBasicSetStruct.ucParity)
                {
                    case 0: // None
                        cboParity_GS.SelectedIndex = 0;
                        break;
                    case 1: // Odd
                        cboParity_GS.SelectedIndex = 1;
                        break;
                    case 2: // Even
                        cboParity_GS.SelectedIndex = 2;
                        break;
                    default:
                        break;
                }
                
                

                // Get stopbits
                switch(mRtuBasicSetStruct.ucStop)
                {
                    case 1: // stopbits 1
                        cboStopBits_GS.SelectedIndex = 0;
                        break;
                    case 2: // stopbits 2
                        cboStopBits_GS.SelectedIndex = 1;
                        break;
                    default:
                        break;
                }
                
                

                // Get Modbus Net ID
                cboMBNetID2_GS.SelectedIndex = mRtuBasicSetStruct.ucNetID - 1;


                // Modbus response delay time (ms)
                txtMBRespDelayTime.Text = mRtuBasicSetStruct.usMBRespDlyT.ToString();

                // Get Gaptime of IR learning (ms)
                txtGapTime.Text = mRtuBasicSetStruct.usGapTime.ToString();

                MessageBox.Show("Get Basic Settings OK", "OK");
            }
            else
            {
                MessageBox.Show("Error in IR_GetBasicSetting()", "Fail");
            }
        }



        private void btnMRTUSetBasicSettings_Click(object sender, RoutedEventArgs e)
        {
            int rtn = 0;

            IR_API_Class.BasicSetParams mRtuBasicSetStruct = new IR_API_Class.BasicSetParams();

            //
            //_comPortNo = (Byte)(cboCOMPort.SelectedIndex + 1);

            // Set baud rate
            switch (cboBaudRate_GS.SelectedIndex)
            {
                case 0:  // 9600 bps
                    _baudRateIndex = 6;
                    break;

                case 1:  // 19200 bps
                    _baudRateIndex = 7;
                    break;

                case 2:  // 38400 bps
                    _baudRateIndex = 8;
                    break;

                case 3:  // 57600 bps
                    _baudRateIndex = 9;
                    break;

                case 4:  // 115200 bps
                    _baudRateIndex = 10;
                    break;
                default:
                    _baudRateIndex = 10;
                    break;
            }

            // Get Data bits
            switch (cboDataBits_GS.SelectedIndex)
            {
                case 0:
                    _databits = (Byte)8;
                    break;
                default:
                    _databits = (Byte)8;
                    break;
            }


            // Get parity
            switch (cboParity_GS.SelectedIndex)
            {
                case 0:  // None
                    _parity = (Byte)0;
                    break;
                case 1:  // Odd
                    _parity = (Byte)1;
                    break;
                case 2:  // Even
                    _parity = (Byte)2;
                    break;
                default:
                    _parity = (Byte)0;
                    break;
            }


            // Get Stip bits index
            switch (cboStopBits_GS.SelectedIndex)
            {
                case 0: // stop bits = 1;
                    _stopbitsIndex = (Byte)1;   
                    break;
                case 1: // stop bits = 2
                    _stopbitsIndex = (Byte)2;   
                    break;
                default:
                    _stopbitsIndex = (Byte)1;
                    break;
            }



            // Get Modbus Net ID
            _mbID = Byte.Parse(cboMBNetID2_GS.Text);


            // *******************************************************************************************
            mRtuBasicSetStruct.ucBaudrateIndex = _baudRateIndex;  // 
            mRtuBasicSetStruct.ucData = _databits;                // Databits = 8
            mRtuBasicSetStruct.ucParity = _parity;                // Parity = 0 (None)
            mRtuBasicSetStruct.ucStop = _stopbitsIndex;           // Stpo bits = 0 (=>1) 
            mRtuBasicSetStruct.usGapTime = UInt16.Parse(txtGapTime.Text);       // GapTime = 150 ms
            mRtuBasicSetStruct.ucNetID = _mbID;                   // MB Net ID = 1       
            mRtuBasicSetStruct.usMBRespDlyT = UInt16.Parse(txtMBRespDelayTime.Text);       // Modbus repsonse delay time = 3 ms

            // Send Basic Settings to IR module.
            rtn = IR_API_Class.IR_SetBasicSetting(_comPortNo, (Byte)(cboMBNetID2.SelectedIndex + 1), mRtuBasicSetStruct);  

            txtSetBasicSet.Text = rtn.ToString();

            if (rtn == IR_API_Class.IR_E_OK)
            {
                MessageBox.Show("Set OK!\n Please close COM and open COM in new Settings!!!", "OK");
            }
            else
            {
                MessageBox.Show("Set error", "Error");
            }

        }



        private void txtBasicSettingDefault_Click(object sender, RoutedEventArgs e)
        {
            cboBaudRate_GS.SelectedIndex = 4;
            cboParity_GS.SelectedIndex = 0;
            cboDataBits_GS.SelectedIndex = 0;
            cboStopBits_GS.SelectedIndex = 0;
            cboMBNetID2_GS.SelectedIndex = 0;
            txtMBRespDelayTime.Text = "3";
            txtGapTime.Text = "80";
        }



        private void btnResetModule_Click(object sender, RoutedEventArgs e)
        {
            int rtn = 0;
            rtn = IR_API_Class.IR_ResetIRModule(_comPortNo);
            switch(rtn)
            {
                case IR_API_Class.IR_E_RESET_OK:
                    txtResetModuleRtn.Text = "Reset communication settings to default temporarily.";
                    break;
                case IR_API_Class.IR_E_RESET_COM_ERR:
                    txtResetModuleRtn.Text = "COM Error.";
                    break;
                case IR_API_Class.IR_E_RESET_MODULE_FAIL:
                    txtResetModuleRtn.Text = "Reset Failed.";
                    break;
                default:
                    break;
            }
            
        }

















        private void btnGetIRCmdSummaryBuff_Click(object sender, RoutedEventArgs e)
        {
            int rtn = 0;
            ushort moduleIndex = 0x01;    // 0x01: IR-210
            // Module index:
            //   0x01 => IR-210
            //   0x11 => IR-712A
            //   0x21 => IR-712-MTCP

            ushort[] dataArr = new ushort[13];
            string dataStr = String.Empty;

            rtn = IR_API_Class.IR_GetIRCmdSummaryBuffer(moduleIndex, dataArr, (ushort)dataArr.Length);

            // Show the IR command summary.
            for (int i = 0; i < dataArr.Length; i++)
            {
                if (i == (dataArr.Length - 1))
                {
                    dataStr += (dataArr[i].ToString());
                    //dataStr += (dataArr[i].ToString("X4"));
                }
                else
                {
                    dataStr += (dataArr[i].ToString() + "-");
                    //dataStr += (dataArr[i].ToString("X4") + "-");
                }

                IrCmdSummaryArr[i] = dataArr[i];
            }

            txtDeviceQty_G.Text = dataArr[0].ToString();
            IrDeviceTotalQty = dataArr[0];
            txtIRCmdQtyTotal_G.Text = dataArr[1].ToString();
            IrCmdCurrTotalCmd = dataArr[1];
            txtIRCmdQtyDevice1_G.Text = dataArr[2].ToString();
            txtIRCmdQtyDevice2_G.Text = dataArr[3].ToString();
            txtIRCmdQtyDevice3_G.Text = dataArr[4].ToString();
            txtIRCmdQtyDevice4_G.Text = dataArr[5].ToString();
            txtIRCmdQtyDevice5_G.Text = dataArr[6].ToString();
            txtIRCmdQtyDevice6_G.Text = dataArr[7].ToString();
            txtIRCmdQtyDevice7_G.Text = dataArr[8].ToString();
            txtIRCmdQtyDevice8_G.Text = dataArr[9].ToString();
            txtIRCmdQtyDevice9_G.Text = dataArr[10].ToString();
            txtIRCmdQtyDevice10_G.Text = dataArr[11].ToString();
            txtIRCmdQtyDevice11_G.Text = dataArr[12].ToString();

            MessageBox.Show(dataStr, "Get dataArr");
        }



        private void btnSetIRCmdSummaryBuff_Click(object sender, RoutedEventArgs e)
        {
            int rtn = 0;
            ushort moduleIndex = 0x01;    // 0x01: IR-210
            // Module index:
            // 0x01 => IR-210
            // 0x11 => IR-712A
            // 0x21 => IR-712-MTCP
            ushort[] dataArr = new ushort[13];
            string dataStr = String.Empty;

            dataArr[0] = UInt16.Parse(txtDeviceQty.Text);
            IrDeviceTotalQty = dataArr[0];
            dataArr[1] = UInt16.Parse(txtIRCmdQtyTotal.Text);
            IrCmdCurrTotalCmd = dataArr[1];
            dataArr[2] = UInt16.Parse(txtIRCmdQtyDevice1.Text);
            dataArr[3] = UInt16.Parse(txtIRCmdQtyDevice2.Text);
            dataArr[4] = UInt16.Parse(txtIRCmdQtyDevice3.Text);
            dataArr[5] = UInt16.Parse(txtIRCmdQtyDevice4.Text);
            dataArr[6] = UInt16.Parse(txtIRCmdQtyDevice5.Text);
            dataArr[7] = UInt16.Parse(txtIRCmdQtyDevice6.Text);
            dataArr[8] = UInt16.Parse(txtIRCmdQtyDevice7.Text);
            dataArr[9] = UInt16.Parse(txtIRCmdQtyDevice8.Text);
            dataArr[10] = UInt16.Parse(txtIRCmdQtyDevice9.Text);
            dataArr[11] = UInt16.Parse(txtIRCmdQtyDevice10.Text);
            dataArr[12] = UInt16.Parse(txtIRCmdQtyDevice11.Text);


            rtn = IR_API_Class.IR_SetIRCmdSummaryBuffer(moduleIndex, dataArr, (ushort)dataArr.Length);

            for (int i = 0; i < dataArr.Length; i++)
            {
                dataStr += (dataArr[i].ToString("X4") + " ");

                IrCmdSummaryArr[i] = dataArr[i];
            }

            MessageBox.Show(dataStr, "Set IR Cmd Summary to the library buffer.");
        }



        private void btnGetIrDevCmdNameInBuffer_Click(object sender, RoutedEventArgs e)
        {
            // Note: The IR learning data buffer of the API library was initilized 
            //       with 3 IR device names and 6 IR command names by IR_SetIrDevCmdNameInBuffer()  
            //       in the IRMRTUUtilWin0_Loaded() event handler.

            int rtn = 0;
            Byte[] IrDevNameArr = new Byte[100];
            Byte[] IrCmdNameArr = new Byte[100];
            UInt16 IrDevNameLength = 0;
            UInt16 IrCmdNameLength = 0;
            rtn = IR_API_Class.IR_GetIrDevCmdNameInBuffer((UInt16)(cboIRCmdNo2.SelectedIndex + 1), IrDevNameArr, (UInt16)IrDevNameArr.Length, ref IrDevNameLength,
                   IrCmdNameArr, (UInt16)IrCmdNameArr.Length, ref IrCmdNameLength);

            txtIRDevNameGet.Text = Encoding.UTF8.GetString(IrDevNameArr, 0, IrDevNameLength);
            txtIRCmdNameGet.Text = Encoding.UTF8.GetString(IrCmdNameArr, 0, IrCmdNameLength);

        }


        private void btnSetIrDevCmdNameInBuffer_Click(object sender, RoutedEventArgs e)
        {
            int rtn = 0;
            Byte[] IrDevNameArrTemp = Encoding.UTF8.GetBytes(txtIRDevNameSet.Text);
            Byte[] IrCmdNameArrTemp = Encoding.UTF8.GetBytes(txtIRCmdNameSet.Text);

            rtn = IR_API_Class.IR_SetIrDevCmdNameInBuffer(
                (UInt16)(cboIRCmdNo2.SelectedIndex + 1),
                IrDevNameArrTemp, (UInt16)IrDevNameArrTemp.Length,
                IrCmdNameArrTemp, (UInt16)IrCmdNameArrTemp.Length
                );
        }



        private void cboDeviceName_DropDownClosed(object sender, EventArgs e)
        {
            ComboBox cbo = sender as ComboBox;
            int index = 0;

            for (int i = 0; i < cbo.SelectedIndex; i++)
            { index += IrCmdSummaryArr[i + 2]; }

            cboIrCmdNameArr.SelectedIndex = index;   // Show the first IR cmd name for the IR device #.
        }



        private void cboIrCmdNameArr_DropDownClosed(object sender, EventArgs e)
        {
            int rtn = 0;
            rtn = IR_API_Class.IR_ChkIrDataExistInBuffer((UInt16)(cboIrCmdNameArr.SelectedIndex + 1));


            if (rtn == IR_API_Class.IR_E_IRCMD_DATA_EXIST)
            {
                Rect1.Fill = new SolidColorBrush(System.Windows.Media.Colors.Red);
            }
            else
            {
                Rect1.Fill = new SolidColorBrush(System.Windows.Media.Colors.White);
            }
        }



        private void cboIrCmdNameArr_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            int rtn = 0;
            rtn = IR_API_Class.IR_ChkIrDataExistInBuffer((UInt16)(cboIrCmdNameArr.SelectedIndex + 1));


            if (rtn == IR_API_Class.IR_E_IRCMD_DATA_EXIST)
            {
                Rect1.Fill = new SolidColorBrush(System.Windows.Media.Colors.Red);
            }
            else
            {
                Rect1.Fill = new SolidColorBrush(System.Windows.Media.Colors.White);
            }
        }



        private void btnRefreshIRDevCmdNameInLibBuf_Click(object sender, RoutedEventArgs e)
        {
            int rtn = 0;
            Byte[] IrDevNameArr = new Byte[100];
            Byte[] IrCmdNameArr = new Byte[100];
            UInt16 IrDevNameLength = 0;
            UInt16 IrCmdNameLength = 0;
            UInt16 irCmdNo = 0;

            btnGetIRCmdSummaryBuff_Click(sender, e);    // Get IR cmd summary from the IR library buffer.

            cboDeviceName.Items.Clear();
            cboIrCmdNameArr.Items.Clear();

            // Initialize the items of IR device names & IR cmd name (comments) comboboxes.  
            for (int i = 0; i < IrCmdSummaryArr[0]; i++)
            {
                for (int j = 0; j < IrCmdSummaryArr[i + 2]; j++)
                {

                    rtn = IR_API_Class.IR_GetIrDevCmdNameInBuffer(++irCmdNo, IrDevNameArr, (UInt16)IrDevNameArr.Length, ref IrDevNameLength,
                           IrCmdNameArr, (UInt16)IrCmdNameArr.Length, ref IrCmdNameLength);

                    cboIrCmdNameArr.Items.Add(irCmdNo.ToString() + "_" + Encoding.UTF8.GetString(IrCmdNameArr, 0, IrCmdNameLength));
                }

                cboDeviceName.Items.Add((i + 1) + "_" + Encoding.UTF8.GetString(IrDevNameArr, 0, IrDevNameLength));
            }

            if ((cboDeviceName.Items.Count != 0) && (cboIrCmdNameArr.Items.Count != 0))
            {
                cboDeviceName.SelectedIndex = cboIrCmdNameArr.SelectedIndex = 0;
            }
        }



        private void btnMRtuLrnON_Click(object sender, RoutedEventArgs e)
        {
            int rtn = IR_API_Class.IR_LrnModeSet(_comPortNo, _mbID, (Byte)1);

            if (rtn != IR_API_Class.IR_E_OK)
            {
                MessageBox.Show("IR-Learn-ON failed", "Error");
            }

        }



        private void btnMRtuLrnOFF_Click(object sender, RoutedEventArgs e)
        {
            int rtn = IR_API_Class.IR_LrnModeSet(_comPortNo, _mbID, (Byte)0);


            if (rtn != IR_API_Class.IR_E_OK)
            {
                MessageBox.Show("IR-Learn-OFF failed", "Error");
            }
        }



        private void btnIsLearnOK_Click(object sender, RoutedEventArgs e)
        {
            UInt16 isLearnOK = 0;   // Variable to indicate if learning an IR cmd is completed. 0=>not yet. 1=>completed.
            int rtn = IR_API_Class.IR_IsIRLearnOK(_comPortNo, ref isLearnOK, UInt16.Parse(txtRecvTout.Text));
            txtIsLrnOKRtn.Text = rtn.ToString();
            MessageBox.Show("Is IR-Learning OK? => " + ((isLearnOK == 1) ? "OK" : "Not OK"), "Is IR learning OK?");
        }



        private void btnGetIRCmdAfterLrn_Click(object sender, RoutedEventArgs e)
        {
            int cboCurrSelectedIndex = cboIrCmdNameArr.SelectedIndex;
            int rtn = IR_API_Class.IR_GetIRCmdAfterLearn(_comPortNo, _mbID, (UInt16)(cboCurrSelectedIndex + 1));

            if (rtn == IR_API_Class.IR_E_OK)
            {
                cboIrCmdNameArr_DropDownClosed(sender, e);
                MessageBox.Show("Call IR_GetIRCmdAfterLearn() OK.", "OK");
            }
            else
            {
                MessageBox.Show("Error in calling IR_GetIRCmdAfterLearn()", "Error");
            }
        }



        private void btnRunCmd_Click(object sender, RoutedEventArgs e)
        {
            int rtn = IR_API_Class.IR_RunCommand(_comPortNo, _mbID, (UInt16)(cboIrCmdNameArr.SelectedIndex + 1), IROutputChArr[cboIROutputCh.SelectedIndex]);

            if (rtn != IR_API_Class.IR_E_OK)
            {
                MessageBox.Show("Error in calling IR_RunCommand()", "Error");
            }
        }



        private void btnClearOneIRDataInLib_Click(object sender, RoutedEventArgs e)
        {
            UInt16 moduleIndex = 0x21;   // IR-712-MTCP
            int cboIndex = cboIrCmdNameArr.SelectedIndex;
            int rtn = IR_API_Class.IR_ClearOneIRLrnDataInAPI((UInt16)(cboIndex + 1), moduleIndex);

            if (rtn != IR_API_Class.IR_E_OK)
            {
                MessageBox.Show("IR_ClearOneIRLrnDataInAPI() return error code = " + rtn.ToString(), "IR_ClearOneIRLrnDataInAPI Error");
            }

            cboIrCmdNameArr_DropDownClosed(sender, e);
        }



        private void btnSetIRCmdSummary_Click(object sender, RoutedEventArgs e)
        {
            int rtn = 0;
            UInt16 moduleIndex = 0x01;  // 0x01: ModuleIndex of IR-210
            UInt16[] dataArr = new UInt16[13];

            MessageBoxResult msgRst = MessageBox.Show(
                "Before pressing this button, press the \"btnSetIRCmdSummaryBuff\" button in advance in group Step 4 to update the IR cmd summary if necessary. \nThis button will write the IR command summary in the library buffer \nto the flash memory of the IR module.\nAre you sure to proceed?\n",
                "Warning", MessageBoxButton.YesNo,
                MessageBoxImage.Warning);

            if (msgRst == MessageBoxResult.No)
            {
                return;
            }


            btnSetIRCmdSummaryBuff_Click(sender, e);

            rtn = IR_API_Class.IR_SetIRCmdSummary(_comPortNo, _mbID, moduleIndex);

            //dataArr[0] = Byte.Parse(txtDeviceQty.Text);
            //dataArr[1] = Byte.Parse(txtIRCmdQtyTotal.Text);
            //dataArr[2] = Byte.Parse(txtIRCmdQtyDevice1.Text);
            //dataArr[3] = Byte.Parse(txtIRCmdQtyDevice2.Text);
            //dataArr[4] = Byte.Parse(txtIRCmdQtyDevice3.Text);
            //dataArr[5] = Byte.Parse(txtIRCmdQtyDevice4.Text);
            //dataArr[6] = Byte.Parse(txtIRCmdQtyDevice5.Text);
            //dataArr[7] = Byte.Parse(txtIRCmdQtyDevice6.Text);
            //dataArr[8] = Byte.Parse(txtIRCmdQtyDevice7.Text);
            //dataArr[9] = Byte.Parse(txtIRCmdQtyDevice8.Text);
            //dataArr[10] = Byte.Parse(txtIRCmdQtyDevice9.Text);
            //dataArr[11] = Byte.Parse(txtIRCmdQtyDevice10.Text);
            //dataArr[12] = Byte.Parse(txtIRCmdQtyDevice11.Text);

            //rtn = IR_API_Class.IR_SetIRCmdSummary1(socketHandle, moduleIndex, dataArr, (UInt16)dataArr.Length);



            txtSetIRCmdSummaryRtn.Text = rtn.ToString();

            //MessageBox.Show("a[0]=" + dataArr[0].ToString("X2")
            //              + "\na[1]=" + dataArr[1].ToString("X2")
            //              + "\na[2]=" + dataArr[2].ToString("X2")
            //              + "\na[3]=" + dataArr[3].ToString("X2")
            //              + "\na[4]=" + dataArr[4].ToString("X2")
            //              + "\na[5]=" + dataArr[5].ToString("X2")
            //              + "\na[6]=" + dataArr[6].ToString("X2")
            //              + "\na[7]=" + dataArr[7].ToString("X2")
            //              );
        }



        private void btnDLOneIRDataToModule_Click(object sender, RoutedEventArgs e)
        {
            thDwLDIR = new Thread(new ThreadStart(DownloadIRProcess));
            thDwLDIR.Start();

        }


        void DownloadIRProcess()
        {
            int rtn = 0;
            for (int i = 0; i < IrCmdCurrTotalCmd; i++)
            {
                rtn = IR_API_Class.IR_DLOneIRDataToModule(_comPortNo, _mbID, (UInt16)(i + 1));

                if (rtn != IR_API_Class.IR_E_OK)
                {
                    MessageBox.Show("Failed to download IR cmds", "Download Error");
                    return;
                }

                Dispatcher.BeginInvoke(
                    (Action)(() =>
                    {
                        txtIRCmdQtyCurr.Text = (i + 1).ToString();
                    }
                    ));
                Thread.Sleep(50);
            }

            MessageBox.Show("Download " + IrCmdCurrTotalCmd.ToString() + " IR cmds to the IR modules successfully.", "Download OK");
        }



        private void btnEmitIRCmd_Click(object sender, RoutedEventArgs e)
        {
            int rtn = IR_API_Class.IR_EmitIrSignal(_comPortNo, _mbID, (UInt16)(cboIRCmdNo.SelectedIndex + 1), IROutputChArr[cboIROutputCh2.SelectedIndex]);
        }



        private void btnGetIRCmdSummary_Click(object sender, RoutedEventArgs e)
        {
            int rtn = 0;
            UInt16 moduleIndex = 0x01;  // 0x01: ModuleIndex of IR-210
            UInt16[] dataArr = new UInt16[13];
            //UInt16[] dataArr2 = new UInt16[13];
           // UInt16[] dataArr3 = new UInt16[13];
            String dataStr = String.Empty;

            rtn = IR_API_Class.IR_GetIRCmdSummary(_comPortNo, _mbID, moduleIndex, dataArr, (UInt16)dataArr.Length);
            txtGetIRCmdSummaryRtn.Text = rtn.ToString();

            if (rtn == IR_API_Class.IR_E_OK)
            {

                // Show the IR command summary.
                for (int i = 0; i < dataArr.Length; i++)
                {
                    if (i == (dataArr.Length - 1))
                    {
                        dataStr += (dataArr[i].ToString());
                        //dataStr += (dataArr[i].ToString("X4"));
                    }
                    else
                    {
                        dataStr += (dataArr[i].ToString() + "-");
                        //dataStr += (dataArr[i].ToString("X4") + "-");
                    }

                    IrCmdSummaryArr[i] = dataArr[i];
                }


                txtGetIRCmdSummary.Text = dataStr;

                IrDeviceTotalQty = dataArr[0];
                IrCmdCurrTotalCmd = dataArr[1];

                MessageBox.Show("Get IR Cmd Summary OK!", "Success");

            }
            else
            {
                MessageBox.Show("Failed to Get IR Cmd Summary!", "Error");
            }

        }



        private void btnLDOneIRDataToAPI_Click(object sender, RoutedEventArgs e)
        {
            thLDIR = new Thread(new ThreadStart(LoadIRProcess));
            thLDIR.Start();
        }


        void LoadIRProcess()
        {
            int rtn = 0;
            for (int i = 0; i < IrCmdCurrTotalCmd; i++)
            {
                rtn = IR_API_Class.IR_LDOneIRDataToAPI(_comPortNo, _mbID, (UInt16)(i + 1));

                if (rtn != IR_API_Class.IR_E_OK)
                {
                    MessageBox.Show("Failed to load IR cmds", "load Error");
                    return;
                }

                Dispatcher.BeginInvoke(
                    (Action)(() =>
                    {
                        txtIRCmdQtyCurr.Text = (i + 1).ToString();
                    }
                    ));
                Thread.Sleep(50);
            }

            MessageBox.Show("Load " + IrCmdCurrTotalCmd.ToString() + " IR cmds back to the IR API library successfully.", "Load back OK");
        }



        private void btnSaveIRCmdToFile_Click(object sender, RoutedEventArgs e)
        {
            int rtn = IR_API_Class.IR_SaveIRCmdsToFile(txtFilePath_Save.Text);
            txtSaveIRCmdToFileRtn.Text = rtn.ToString();

            if (rtn != IR_API_Class.IR_E_OK)
            {
                MessageBox.Show("Save failed.");
            }
        }



        private void btnLoadIRCmdToFile_Click(object sender, RoutedEventArgs e)
        {
            int rtn = IR_API_Class.IR_LoadIRCmdsFromFile(txtFilePath_Load.Text);
            txtLoadIRCmdToFileRtn.Text = rtn.ToString();

            if (rtn != IR_API_Class.IR_E_OK)
            {
                MessageBox.Show("Load failed.");
            }
        }



        private void txtIRCmdQtyDevice1_TextChanged(object sender, TextChangedEventArgs e)
        {
            TextBox txt = sender as TextBox;
            int irCmdOtyTotal = 0;
            int irCmdQtyDevice = 0;
            int irDeviceQtyTotal = 0;


            // #1 device - IR cmd qty
            if (txtIRCmdQtyDevice1 == null || int.TryParse(txtIRCmdQtyDevice1.Text, out irCmdQtyDevice) == false)
            {
                irCmdQtyDevice = 0;
            }
            irCmdOtyTotal += irCmdQtyDevice;
            irDeviceQtyTotal += (irCmdQtyDevice > 0 ? 1 : 0);


            // #2 device - IR cmd qty
            if (txtIRCmdQtyDevice2 == null || int.TryParse(txtIRCmdQtyDevice2.Text, out irCmdQtyDevice) == false)
            {
                irCmdQtyDevice = 0;
            }
            irCmdOtyTotal += irCmdQtyDevice;
            irDeviceQtyTotal += (irCmdQtyDevice > 0 ? 1 : 0);

            // #3 device - IR cmd qty
            if (txtIRCmdQtyDevice3 == null || int.TryParse(txtIRCmdQtyDevice3.Text, out irCmdQtyDevice) == false)
            {
                irCmdQtyDevice = 0;
            }
            irCmdOtyTotal += irCmdQtyDevice;
            irDeviceQtyTotal += (irCmdQtyDevice > 0 ? 1 : 0);

            // #4 device - IR cmd qty
            if (txtIRCmdQtyDevice4 == null || int.TryParse(txtIRCmdQtyDevice4.Text, out irCmdQtyDevice) == false)
            {
                irCmdQtyDevice = 0;
            }
            irCmdOtyTotal += irCmdQtyDevice;
            irDeviceQtyTotal += (irCmdQtyDevice > 0 ? 1 : 0);

            // #5 device - IR cmd qty
            if (txtIRCmdQtyDevice5 == null || int.TryParse(txtIRCmdQtyDevice5.Text, out irCmdQtyDevice) == false)
            {
                irCmdQtyDevice = 0;
            }
            irCmdOtyTotal += irCmdQtyDevice;
            irDeviceQtyTotal += (irCmdQtyDevice > 0 ? 1 : 0);

            // #6 device - IR cmd qty
            if (txtIRCmdQtyDevice6 == null || int.TryParse(txtIRCmdQtyDevice6.Text, out irCmdQtyDevice) == false)
            {
                irCmdQtyDevice = 0;
            }
            irCmdOtyTotal += irCmdQtyDevice;
            irDeviceQtyTotal += (irCmdQtyDevice > 0 ? 1 : 0);

            // #7 device - IR cmd qty
            if (txtIRCmdQtyDevice7 == null || int.TryParse(txtIRCmdQtyDevice7.Text, out irCmdQtyDevice) == false)
            {
                irCmdQtyDevice = 0;
            }
            irCmdOtyTotal += irCmdQtyDevice;
            irDeviceQtyTotal += (irCmdQtyDevice > 0 ? 1 : 0);

            // #8 device - IR cmd qty
            if (txtIRCmdQtyDevice8 == null || int.TryParse(txtIRCmdQtyDevice8.Text, out irCmdQtyDevice) == false)
            {
                irCmdQtyDevice = 0;
            }
            irCmdOtyTotal += irCmdQtyDevice;
            irDeviceQtyTotal += (irCmdQtyDevice > 0 ? 1 : 0);

            // #9 device - IR cmd qty
            if (txtIRCmdQtyDevice9 == null || int.TryParse(txtIRCmdQtyDevice9.Text, out irCmdQtyDevice) == false)
            {
                irCmdQtyDevice = 0;
            }
            irCmdOtyTotal += irCmdQtyDevice;
            irDeviceQtyTotal += (irCmdQtyDevice > 0 ? 1 : 0);

            // #10 device - IR cmd qty
            if (txtIRCmdQtyDevice10 == null || int.TryParse(txtIRCmdQtyDevice10.Text, out irCmdQtyDevice) == false)
            {
                irCmdQtyDevice = 0;
            }
            irCmdOtyTotal += irCmdQtyDevice;
            irDeviceQtyTotal += (irCmdQtyDevice > 0 ? 1 : 0);

            // #11 device - IR cmd qty
            if (txtIRCmdQtyDevice11 == null || int.TryParse(txtIRCmdQtyDevice11.Text, out irCmdQtyDevice) == false)
            {
                irCmdQtyDevice = 0;
            }
            irCmdOtyTotal += irCmdQtyDevice;
            irDeviceQtyTotal += (irCmdQtyDevice > 0 ? 1 : 0);

            txtIRCmdQtyTotal.Text = irCmdOtyTotal.ToString();
            txtDeviceQty.Text = irDeviceQtyTotal.ToString();
        }



        private void btnGetMBRecvTOut_Click(object sender, RoutedEventArgs e)
        {
            int rtn = IR_API_Class.IR_GetModbusRecvTimeout();
            txtRecvTout1.Text = rtn.ToString();
        }

        

        private void btnSetMBRecvTOut_Click(object sender, RoutedEventArgs e)
        {
            IR_API_Class.IR_SetModbusRecvTimeout((UInt16)int.Parse(txtRecvTout1.Text));
        }


    }
}
