﻿' ******************************************************************************************
' VB Wrapper of the IR API Library
'   
' Module name: IR_API_Class
' 
' Description:
'   VB wrapper module of the IR API library for IR learning remote module (IR-210/IR-712A/IR-712-MTCP).
'
' Release Note:
'   v2.2.0.0 [2022-June-02]
' ****************************************************************************************** 


Imports System
Imports System.Runtime.InteropServices

Module IR_API_Class


    '
    ' Return Code
    '
    Public Const IR_E_OK As Integer = &H0                           ' No errors. The function runs successfully.                                                                                      
    Public Const IR_E_COM_STATUS_AVAIL As Integer = &H1             ' The COM port is available and not open.                                                                                         
    Public Const IR_E_IRCMD_DATA_EMPTY As Integer = &H0             ' NO IR data exist for the specified IR cmd number in the API buffer.                                                             
    Public Const IR_E_IRCMD_DATA_EXIST As Integer = &H1             ' IR data exist for the specified IR cmd number in the API buffer.                                                                
    Public Const IR_E_RESET_OK As Integer = &H2                     ' Reset IR module sucessfully.                                                                                                    

    Public Const IR_E_FUNC_ERR As Integer = &H11                    ' Function error.                                                                                                                 
    Public Const IR_E_COM_STATUS_ERR As Integer = &H12              ' The COM port is not available or is already open.                                                                               
    Public Const IR_E_OPEN_COM_ERR As Integer = &H13                ' Open COM port error                                                                                                             
    Public Const IR_E_ILL_BAUD_IDX As Integer = &H14                ' Illegal baud rate index                                                                                                         
    Public Const IR_E_ILL_DATA_BIT As Integer = &H15                ' Illegal data bits                                                                                                               
    Public Const IR_E_ILL_STOP_BIT As Integer = &H16                ' Illegal stop bits                                                                                                               
    Public Const IR_E_ILL_PARITY As Integer = &H17                  ' Illegal parity                                                                                                                  
    Public Const IR_E_MODNAME_INVALID As Integer = &H18             ' Get invalid module name.                                                                                                        
    Public Const IR_E_SEND_MBCMD_ERR As Integer = &H19              ' Error in sending/receiving Modbus command.                                                                                      
    Public Const IR_E_SEND_MBCMD_FLASH_ERR As Integer = &H1A        ' Error in sending/receiving Modbus command to write data to Flash.                                                               
    Public Const IR_E_ILL_SUMMARY_LEN As Integer = &H1B             ' Illegal length of the array for IR command summary (should be 13)                                                               
    Public Const IR_E_DEVICE_QTY_ERR As Integer = &H1C              ' The device quantity exceeds the maximum.                                                                                        
    Public Const IR_E_IRCMD_TOTAL_QTY_ERR As Integer = &H1D         ' The element "Total quantity of IR commands" in the IR cmd summary exceeds the maximum.                                          
    Public Const IR_E_IRCMD_SUM_QTY_ERR As Integer = &H1E           ' The sum of all device's IR command quantity exceeds the maximum.                                                                
    Public Const IR_E_ILL_MODULE_INDEX As Integer = &H1F            ' Illegal module index                                                                                                            
    Public Const IR_E_ILL_IRCMD_NO As Integer = &H20                ' Illegal IR command number                                                                                                       
    Public Const IR_E_DEVICE_NAME_ARR_UNDERSIZE As Integer = &H21   ' The length of the device name array is too small as an argument to a function.                                                  
    Public Const IR_E_IRCMD_NAME_ARR_UNDERSIZE As Integer = &H22    ' The length of the IR cmd name array is too small as an argument to a function.                                                  
    Public Const IR_E_DEVICE_NAME_ARR_OVERSIZE As Integer = &H23    ' The length of the device name is too large as an argument to a function.                                                        
    Public Const IR_E_IRCMD_NAME_ARR_OVERSIZE As Integer = &H24     ' The length of the IR cmd name is too large as an argument to a function.                                                        
    Public Const IR_E_LRNCMD_LEN_EXCEED_MAX As Integer = &H25       ' The length of the IR cmd learning data exceeds the maximum.                                                                     
    Public Const IR_E_RESET_COM_ERR As Integer = &H26               ' COM error in sending reset message to the IR module.                                                                            
    Public Const IR_E_RESET_MODULE_FAIL As Integer = &H27           ' Resetting the IR module is not successful.                                                                                      
    Public Const IR_E_IRCMD_TOTALQTY_NOMATCH As Integer = &H28      ' The totl quantity of IR cmds do not match the sum of IR cmds in all devices in the array of IR cmd summary                      
    Public Const IR_E_OPEN_FILE_ERROR As Integer = &H29             ' Open file error																		                                           
    Public Const IR_E_CLOSE_FILE_ERROR As Integer = &H2A            ' Close file error																		                                           
    Public Const IR_E_IR_FILE_FORMAT_ERR As Integer = &H2B          ' Format error in IR learning data file												                                           
    Public Const IR_E_SOCKET_INVALID As Integer = (-1)              ' SOCKET Error																			                                           
    Public Const IR_E_SOCKET_BIND_ERR As Integer = &H2C             ' Error in binding socket with the network address of NIC                                                                         
    Public Const IR_E_SOCKET_CONN_TOUT As Integer = &H2D            ' Connection timeout by Modbus TCP                                                                                                
    Public Const IR_E_SOCKET_RECV_TOUT As Integer = &H2E            ' Socket receive timeout                                                                                                          



    ' Structure definition for the parameter of function IR_Get/SetBasicSetting()
    ' This structure colllects basic setting parameters of IR modules with Modbus RTU comminucation.
    <StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Ansi, Pack:=1)>
    Public Structure BasicSetParams
        <MarshalAs(UnmanagedType.U1)> Public ucBaudrateIndex As Byte  ' Baud rate Index. [6]:9600 bps, [10]:115200 bps
        <MarshalAs(UnmanagedType.U1)> Public ucData As Byte           ' Data bits  always 8
        <MarshalAs(UnmanagedType.U1)> Public ucParity As Byte         ' Parity  0=>None  1=>Odd  2=>Even
        <MarshalAs(UnmanagedType.U1)> Public ucStop As Byte           ' Stop bits.  1 => 1 stopbit,  2 => 2 stopbits
        <MarshalAs(UnmanagedType.U1)> Public ucNetID As Byte          ' Modbus Net ID of IR modules ( 1 ~ 247 )
        <MarshalAs(UnmanagedType.U2)> Public usMBRespDlyT As UShort   ' Modbus response delay time (ms), default = 3 ms
        <MarshalAs(UnmanagedType.U2)> Public usGapTime As UShort      ' Gap time (ms), default= 72 ms, range = 72 ~ 200 ms
    End Structure


    ' Structure definition for the parameter of function IR_MTCP_Get/SetBasicSetting() & IR_MUDP_Get/SetBasicSetting().
    ' This structure colllects basic setting parameters of IR modules with Modbus TCP/UDP communication.
    <StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Ansi, Pack:=1)>
    Public Structure BasicSetParamsMTCP
        <MarshalAs(UnmanagedType.U4)> Public ulIPAddr As UInteger       ' IP Addres.   192.168.0.1 = > 0xC0A80001
        <MarshalAs(UnmanagedType.U4)> Public ulMaskAddr As UInteger     ' Mask Addres.  255.255.0.0 = > 0xFFFF0000
        <MarshalAs(UnmanagedType.U4)> Public ulGWAddr As UInteger       ' Gateway address. 192.168.0.254 = > 0xC0A800FE
        <MarshalAs(UnmanagedType.U4)> Public ulIdleTOut As UInteger     ' Idle communication timeout (mimnute)
        <MarshalAs(UnmanagedType.U1)> Public ucNetID As Byte            ' Modbus Net ID of IR modules ( 1 ~ 247 )
        <MarshalAs(UnmanagedType.U1)> Public ucAddrType As Byte         ' Address type. 0: static  1: DHCP
        <MarshalAs(UnmanagedType.U2)> Public usGapTime As UShort        ' Gap time (ms), default= 72 ms, range = 72 ~ 200 ms
    End Structure



    ' Structure definition for the parameter of function IR_EthGetSearchModuleInfo()
    ' This structure colllects parameters responsed from IR modules with Ethernet interface.
    <StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Ansi, Pack:=1)>
    Public Structure SearchRespEth
        <MarshalAs(UnmanagedType.U8)> Public dwlMacAddr As ULong      ' MAC address 1a:2b:3c:4d:5e:6f => 0x00001A2B3C4D5E6F
        <MarshalAs(UnmanagedType.U4)> Public ulIPAddr As UInteger     ' IP Addres.   192.168.0.1 => 0xC0A80001
        <MarshalAs(UnmanagedType.U4)> Public ulMaskAddr As UInteger   ' Mask Addres.  255.255.0.0 => 0xFFFF0000
        <MarshalAs(UnmanagedType.U4)> Public ulGWAddr As UInteger     ' Gateway address. 192.168.0.254 => 0xC0A800FE 
        <MarshalAs(UnmanagedType.U4)> Public ulFwVer As UInteger      ' Firmware version of the module. v1.2.3 => 0x00010203 
        <MarshalAs(UnmanagedType.U4)> Public ulIdleTOut As UInteger   ' Idle communication timeout (unit: mimnute, 0 ~ 65535)
        <MarshalAs(UnmanagedType.U1)> Public ucNetID As Byte          ' Modbus Net ID of IR modules ( 1 ~ 247 ) 
        <MarshalAs(UnmanagedType.U1)> Public ucAddrType As Byte       ' Address type. 0: static  1: DHCP
        <MarshalAs(UnmanagedType.U1)> Public ucHwMode As Byte         ' Mode: OP(=0) or FW/Init (=1) mode
        <MarshalAs(UnmanagedType.U1)> Public ucModIndex As Byte       ' Module Index.   0x21 => "IR-712-MTCP"	
    End Structure






    Public Declare Auto Function MBox Lib "user32.dll" Alias "MessageBox" (
        ByVal hWnd As Integer,
        ByVal txt As String,
        ByVal caption As String,
        ByVal Typ As Integer) As Integer


    ' @1
    'Public Declare Auto Function c Lib "irlrnapi.dll" () As UInteger
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_GetAPIVersion() As UInteger
    End Function



    ' =================
    ' @2
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_GetAPIDate() As UInteger
    End Function



    ' @3
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_GetComPortStatus(
            ByVal ucPort As Byte
            ) As Integer
    End Function




    ' @4
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_OpenCom(
            ByVal comPortNo As Byte,
            ByVal ucBaudIndex As Byte,
            ByVal ucData As Byte,
            ByVal ucParity As Byte,
            ByVal ucStop As Byte
            ) As Integer
    End Function



    ' @5
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_CloseCom(
            ByVal ucPort As Byte
            ) As Integer
    End Function



    ' @6
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_SetBasicSetting(
            ByVal ucPort As Byte,
            ByVal ucNetID As Byte,
            ByVal SBasicSetting As BasicSetParams
            ) As Integer
    End Function


    ' @7
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_GetBasicSetting(
            ByVal ucPort As Byte,
            ByVal ucNetID As Byte,
            ByRef SBasicSetting As BasicSetParams
            ) As Integer
    End Function






    ' @8
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_ChkModuleName(
        ByVal ucPort As Byte,
        ByVal ucNetID As Byte,
        ByRef pwModuleIndex As UShort
        ) As Integer
    End Function


    ' @9
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_LrnModeSet(
        ByVal ucPort As Byte,
        ByVal ucNetID As Byte,
        ByVal ucMode As Byte
        ) As Integer
    End Function


    ' @10
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_IsIRLearnOK(
        ByVal ucPort As Byte,
        ByRef pusIsOK As UShort,
        ByVal usTimeout As UShort
        ) As Integer
    End Function

    ' @11
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_SetIRCmdSummary(
        ByVal ucPort As Byte,
        ByVal usNetID As Byte,
        ByVal usIrModuleIndex As UShort
        ) As Integer
    End Function

    ' @30  'not tested yet
    ' ref link: https://msdn.microsoft.com/zh-tw/library/z6cfh6e6(v=vs.110).aspx?cs-save-lang=1&cs-lang=vb#code-snippet-4
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_SetIRCmdSummary1(
        ByVal ucPort As Byte,
        ByVal usNetID As Byte,
        ByVal usIrModuleIndex As UShort,
        <MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=4)> dataArr As UShort(),
        ByVal usInElementCnt As UShort
        ) As Integer
    End Function

    ' @12  ' Not tested yet  , 
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_GetIRCmdSummary(
        ByVal ucPort As Byte,
        ByVal usNetID As Byte,
        ByVal usIrModuleIndex As UShort,
        <MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=4)> dataArr As UShort(),
        ByVal usInElementCnt As UShort
        ) As Integer
    End Function

    ' @13
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_GetIRCmdAfterLearn(
        ByVal ucPort As Byte,
        ByVal usNetID As Byte,
        ByVal usIrCmdNumber As UShort
        ) As Integer
    End Function

    ' @14
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_ChkIrDataExistInBuffer(
        ByVal usIrCmdNumber As UInt16
        ) As UInt32
    End Function

    'Public Declare Auto Function IR_GetIrDevCmdNameInBuffer Lib "irlrnapi.dll" (
    '                ByVal usIrCmdNumber As UShort,
    '                <MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=2)> cIrDevNameStr() As Byte,
    '                ByVal ucDevNameUcLen As UShort,
    '                ByRef usDevNameStrByteLen As UShort,
    '                <MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=5)> cIrCmdNameStr() As Byte,
    '                ByVal ucIrCmdNameUcLen As UShort,
    '                ByRef usCmdNameStrByteLen As UShort) As UShort

    '@15
    Public Declare Auto Function IR_GetIrDevCmdNameInBuffer Lib "irlrnapi.dll" (
                ByVal usIrCmdNumber As UShort,
                <MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=2)> ucIrDevNameStr As Byte(),
                ByVal ucDevNameUcLen As UShort,
                ByRef usDevNameStrByteLen As UShort,
                <MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=5)> ucIrCmdNameStr As Byte(),
                ByVal ucIrCmdNameUcLen As UShort,
                ByRef usCmdNameStrByteLen As UShort
                ) As Integer




    'Public Declare Auto Function IR_SetIrDevCmdNameInBuffer Lib "irlrnapi.dll" (
    '                ByVal usIrCmdNumber As UShort,
    '                <MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=2)> cIrDevNameStr() As Byte,
    '                ByVal ucDevNameUcLen As Byte,
    '                <MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=4)> cIrCmdNameStr() As Byte,
    '                ByVal ucIrCmdNameUcLen As Byte) As UShort

    '@16
    Public Declare Auto Function IR_SetIrDevCmdNameInBuffer Lib "irlrnapi.dll" (
                ByVal usIrCmdNumber As UShort,
                <MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=2)> ucIrDevNameStr As Byte(),
                ByVal usDevNameUcLen As UShort,
                <MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=4)> ucIrCmdNameStr As Byte(),
                ByVal usIrCmdNameUcLen As UShort
                ) As Integer


    ' @17
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_EmitIrSignal(
        ByVal ucPort As Byte,
        ByVal ucNetID As Byte,
        ByVal usIrCmdNumber As UInt16,
        ByVal ucIrOutputCh As Byte
        ) As Int32
    End Function


    ' @18		
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_RunCommand(
        ByVal ucPort As Byte,
        ByVal ucNetID As Byte,
        ByVal usIrCmdNumber As UInt16,
        ByVal ucIrOutputCh As Byte
        ) As Int32
    End Function


    ' @19
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_DLOneIRDataToModule(
        ByVal ucPort As Byte,
        ByVal ucNetID As Byte,
        ByVal usIrCmdNumber As UInt16
        ) As Int32
    End Function


    ' @20
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_LDOneIRDataToAPI(
        ByVal ucPort As Byte,
        ByVal ucNetID As Byte,
        ByVal usIrCmdNumber As UInt16
        ) As Int32
    End Function


    ' @21
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_ResetIRModule(
        ByVal ucPort As Byte
        ) As Int32
    End Function


    ' @22
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_GetModbusRecvTimeout() As Int32
    End Function


    ' @23
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_SetModbusRecvTimeout(
        ByVal recvTout As UInt16
        ) As Int32
    End Function


    '@24
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_ClearOneIRLrnDataInAPI(
        ByVal usIrCmdNumber As UInt16,
        ByVal usIrModuleIndex As UInt16
        ) As Int32
    End Function


    '@25
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_ClearOneIRDeiveCmdNameInAPI(
        ByVal usIrCmdNumber As UInt16,
        ByVal usIrModuleIndex As UInt16
        ) As Int32
    End Function


    '@26
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_GetIRCmdSummaryBuffer(
            ByVal usIrModuleIndex As UInt16,
            <MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=2)> dataArr As UInt16(),
            ByVal usInElementCnt As UInt16
            ) As Int32
    End Function


    '@27
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_SetIRCmdSummaryBuffer(
            ByVal usIrModuleIndex As UInt16,
            <MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=2)> dataArr As UInt16(),
            ByVal usInElementCnt As UInt16
            ) As Int32
    End Function


    '@28
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_LoadIRCmdsFromFile(
            ByVal filepath As String
            ) As Int32
    End Function


    '@29
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_SaveIRCmdsToFile(
            ByVal filepath As String
            ) As Int32
    End Function


    '@47
    'void __stdcall IR_ChkNIC(
    'int *iNiCnt
    ')
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Sub IR_ChkNIC(
            ByRef iNiCnt As Int32
            )
    End Sub



    '@48
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Sub IR_GetNICInfo(
            ByVal ucIndex As Byte,
            <MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=2)> szDescStr As Byte(),
            ByVal usDescStrLen As UInt16,
            ByRef ucRtnDecrLen As UInt16,
            ByRef ulIPAddr As UInt32
            )
    End Sub


    '@49
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_EthSearchModules(
            ByVal ulIPAddrNIC As UInt32,
            ByRef pucModuleCnt As Byte,
            ByVal usWaitTimeout As UInt16
            ) As Int32
    End Function


    '@50
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Sub IR_EthGetSearchModuleInfo(
            ByVal usCntIndex As UInt16,
            ByRef ModuleInfo As SearchRespEth
            )
    End Sub


    '@60
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MTCP_TCPNew() As Int32
    End Function



    '@61
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MTCP_TCPNew1(
            ByVal ulNicIPAddr As UInt32
            ) As Int32
    End Function



    '@62
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MTCP_TCPClose(
           ByVal hSocketTCP As Int32
           ) As Int32
    End Function



    '@63
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MTCP_Connect(
           ByVal hSocketTCP As Int32,
           ByVal ulIPAddr As UInt32,
           ByVal ucNetID As Byte,
           ByVal timeoutMs As UInt16
           ) As Int32
    End Function


    '@69
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MTCP_GetModbusRecvTimeout(
            ) As Int32
    End Function


    '@70
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Sub IR_MTCP_SetModbusRecvTimeout(
            ByVal recvTout As UInt16
            )
    End Sub



    '@71
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MTCP_GetBasicSetting(
            ByVal hSocketTCP As Int32,
            ByRef pEBasicSettingMTCP As BasicSetParamsMTCP
            ) As Int32
    End Function



    '@72
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MTCP_SetBasicSetting(
            ByVal hSocketTCP As Int32,
            ByVal EBasicSettingMTCP As BasicSetParamsMTCP,
            ByVal ucReboot As Byte
            ) As Int32
    End Function


    '@73
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MTCP_RebootIRModule(
            ByVal hSocketTCP As Int32
            ) As Int32
    End Function


    '@74
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MTCP_LrnModeSet(
            ByVal hSocketTCP As Int32,
            ByVal ucMode As Byte
            ) As Int32
    End Function


    '@75
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MTCP_IsIRLearnOK(
            ByVal hSocketTCP As Int32,
            ByRef pusIsOK As UInt16,
            ByVal sTimeout As Int32
            ) As Int32
    End Function



    '@76
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MTCP_GetIRCmdAfterLearn(
            ByVal hSocketTCP As Int32,
            ByVal usIrCmdNumber As UInt16
            ) As Int32
    End Function



    '@77
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MTCP_EmitIrSignal(
            ByVal hSocketTCP As Int32,
            ByVal usIrCmdNumber As UInt16,
            ByVal ucIrOutputCh As Byte
            ) As Int32
    End Function



    '@78
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MTCP_RunCommand(
            ByVal hSocketTCP As Int32,
            ByVal usIrCmdNumber As UInt16,
            ByVal ucIrOutputCh As Byte
            ) As Int32
    End Function


    '@79
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MTCP_DLOneIRDataToModule(
            ByVal hSocketTCP As Int32,
            ByVal usIrCmdNumber As UInt16
            ) As Int32
    End Function


    '@81
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MTCP_LDOneIRDataToAPI(
            ByVal hSocketTCP As Int32,
            ByVal usIrCmdNumber As UInt16
            ) As Int32
    End Function


    '@82
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MTCP_ChkModuleName(
            ByVal hSocketTCP As Int32,
            ByRef pwModuleIndex As UInt16
            ) As Int32
    End Function


    '@83
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MTCP_GetIRCmdSummary(
            ByVal hSocketTCP As Int32,
            ByVal usIrModuleIndex As UInt16,
            <MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=3)> dataArr As UInt16(),
            ByVal usInElementCnt As Int16
            ) As Int32
    End Function


    '@84
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MTCP_SetIRCmdSummary(
            ByVal hSocketTCP As Int32,
            ByVal usIrModuleIndex As UInt16
            ) As Int32
    End Function


    '@85
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MTCP_SetIRCmdSummary1(
            ByVal hSocketTCP As Int32,
            ByVal usIrModuleIndex As UInt16,
            <MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=3)> dataArr As UInt16(),
            ByVal usInElementCnt As Int16
            ) As Int32
    End Function


    ' *************************************
    '  Modbus UDP Communication Functions
    ' *************************************

    '@100
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MUDP_UDPNew(
            ByVal ulIPAddr As UInt32,
            ByVal ucNetID As Byte
            ) As Int32
    End Function


    '@101
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MUDP_UDPNew1(
            ByVal ulNicIPAddr As UInt32,
            ByVal ulIPAddr As UInt32,
            ByVal ucNetID As Byte
            ) As Int32
    End Function


    '@102
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MUDP_UDPClose(
           ByVal hSocketUDP As Int32
           ) As Int32
    End Function


    '@106
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MUDP_GetBasicSetting(
            ByVal hSocketUDP As Int32,
            ByRef pEBasicSettingMUDP As BasicSetParamsMTCP
            ) As Int32
    End Function



    '@107
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MUDP_SetBasicSetting(
            ByVal hSocketUDP As Int32,
            ByVal EBasicSettingMUDP As BasicSetParamsMTCP,
            ByVal ucReboot As Byte
            ) As Int32
    End Function


    '@108
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MUDP_RebootIRModule(
            ByVal hSocketUDP As Int32
            ) As Int32
    End Function


    '@109
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MUDP_GetModbusRecvTimeout() As Int32
    End Function

    '@110
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MUDP_SetModbusRecvTimeout(
            ByVal recvTout As UInt16
            ) As Int32
    End Function


    '@111
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MUDP_LrnModeSet(
            ByVal hSocketUDP As Int32,
            ByVal ucMode As Byte
            ) As Int32
    End Function


    '@112
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MUDP_IsIRLearnOK(
            ByVal hSocketUDP As Int32,
            ByRef pusIsOK As UInt16,
            ByVal sTimeout As Int32
            ) As Int32
    End Function


    '@113
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MUDP_EmitIrSignal(
            ByVal hSocketUDP As Int32,
            ByVal usIrCmdNumber As UInt16,
            ByVal ucIrOutputCh As Byte
            ) As Int32
    End Function


    '@114
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MUDP_RunCommand(
            ByVal hSocketUDP As Int32,
            ByVal usIrCmdNumber As UInt16,
            ByVal ucIrOutputCh As Byte
            ) As Int32
    End Function


    '@115
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MUDP_GetIRCmdAfterLearn(
            ByVal hSocketUDP As Int32,
            ByVal usIrCmdNumber As UInt16
            ) As Int32
    End Function


    '@116
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MUDP_DLOneIRDataToModule(
            ByVal hSocketUDP As Int32,
            ByVal usIrCmdNumber As UInt16
            ) As Int32
    End Function


    '@118
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MUDP_LDOneIRDataToAPI(
            ByVal hSocketUDP As Int32,
            ByVal usIrCmdNumber As UInt16
            ) As Int32
    End Function


    '@119
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MUDP_ChkModuleName(
            ByVal hSocketUDP As Int32,
            ByRef pwModuleIndex As UInt16
            ) As Int32
    End Function


    '@120
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MUDP_GetIRCmdSummary(
            ByVal hSocketUDP As Int32,
            ByVal usIrModuleIndex As Int16,
            <MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=3)> usDataArr As UInt16(),
            ByVal usInElementCnt As UInt16
            ) As Int32
    End Function


    '@121
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MUDP_SetIRCmdSummary(
            ByVal hSocketUDP As Int32,
            ByVal usIrModuleIndex As Int16
            ) As Int32
    End Function


    '@122
    <DllImport("irlrnapi.dll", CallingConvention:=CallingConvention.StdCall)>
    Public Function IR_MUDP_SetIRCmdSummary1(
            ByVal hSocketUDP As Int32,
            ByVal usIrModuleIndex As Int16,
            <MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=3)> usDataArr As UInt16(),
            ByVal usInElementCnt As UInt16
            ) As Int32
    End Function


End Module
